/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.mapgraphic.grid;

import java.awt.Color;


import net.refractions.udig.ui.graphics.ViewportGraphics;

/**
 * Style for the {@link GridMapGraphic}.
 * 
 * @author Jesse
 * @since 1.1.0
 */
public class GridStyle {

    public enum Type {
        SCREEN, WORLD
    }

    public static final String ID = "net.refractions.udig.tool.edit.mapgraphic.grid.style";
    public static final GridStyle DEFAULT_STYLE = new GridStyle(Type.SCREEN, 25, 25, new Color(0,
            0, 255, 100), ViewportGraphics.LINE_DOT, 1);

    private double[] gridSize;
    private Type type;
    private Color color;
    /**
     * One of
     * <ul>
     * <li>{@link ViewportGraphics#LINE_DASH},</li>
     * <li>{@link ViewportGraphics#LINE_DASHDOT}, </li>
     * <li>{@link ViewportGraphics#LINE_DASHDOTDOT}, </li>
     * <li>{@link ViewportGraphics#LINE_DOT}, </li>
     * <li>{@link ViewportGraphics#LINE_SOLID} </li>
     */
    private int lineStyle;
    /**
     * Width of the grid line in pixels
     */
    private int lineWidth;

    public GridStyle( Type type, double dx, double dy, Color color, int lineStyle, int lineWidth ) {
        gridSize = new double[]{dx, dy};
        this.type = type;
        this.color = color;
        this.lineStyle = lineStyle;
        this.lineWidth = lineWidth;
    }

    public GridStyle( GridStyle oldStyle ) {
        gridSize = oldStyle.getGridSize();
        type = oldStyle.getType();
        color = oldStyle.getColor();
        lineStyle = oldStyle.getLineStyle();
        lineWidth = oldStyle.getLineWidth();
    }

    public double[] getGridSize() {
        return gridSize;

    }

    public Type getType() {
        return type;
    }

    public void setType( Type type ) {
        this.type = type;
    }

    public void setGridSize( double dx, double dy ) {
        this.gridSize[0] = dx;
        this.gridSize[1] = dy;
    }

    public Color getColor() {
        return color;
    }

    public void setColor( Color color ) {
        this.color = color;
    }

    public int getLineStyle() {
        return lineStyle;
    }

    public void setLineStyle( int lineStyle ) {
        this.lineStyle = lineStyle;
    }

    public int getLineWidth() {
        return lineWidth;
    }

    public void setLineWidth( int lineWidth ) {
        this.lineWidth = lineWidth;
    }

}
