/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import net.refractions.udig.project.ILayer;

import org.eclipse.core.runtime.IAdaptable;
import org.geotools.factory.CommonFactoryFinder;
import org.geotools.factory.GeoTools;
import org.geotools.filter.AbstractFilter;
import org.opengis.filter.FilterFactory;
import org.opengis.filter.Filter;
import org.opengis.filter.FilterVisitor;

/**
 * Wraps a filter and can adapt to the wrapped filter or to possibly a layer if it is associated with a layer.  
 * For example if it is the selection filter of a layer.
 * 
 * @author jones
 * @since 1.1.0
 */
public class AdaptingFilter implements Filter, IAdaptable {

    final private Filter wrapped;
    protected Set<Object> adapters = new CopyOnWriteArraySet<Object>();
    
    public AdaptingFilter(Filter filter) {
        this( filter, null );
	}
        
    public AdaptingFilter(org.opengis.filter.Filter filter, ILayer source) {
        if( filter==null )
            throw new NullPointerException("filterA cannot be null"); //$NON-NLS-1$
    	wrapped = filter;
	}

    @SuppressWarnings("unchecked")
    public Object getAdapter( Class adapter ) {
        for( Object obj : adapters ) {
            if( adapter.isAssignableFrom(obj.getClass()) )
                return obj;
        }
        if( Filter.class.isAssignableFrom(adapter)){
            return wrapped;
        }
        return null;
    }

    public void addAdapter( Object adapter ) {
        if( adapter==null )
            throw new NullPointerException("adapter cannont be null"); //$NON-NLS-1$
        adapters.add(adapter);
    }

    public boolean removeAdapter( Object adapter ) {
        if( adapter==null )
            throw new NullPointerException("adapter cannont be null"); //$NON-NLS-1$
        return adapters.remove(adapter);
    }

	public Object accept(FilterVisitor visitor, Object extraData) {
		return wrapped.accept(visitor, extraData);
	}

	public boolean evaluate(Object object) {
		return wrapped.evaluate(object);
	}

  
}
