package net.refractions.udig.project.ui.internal;

import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.internal.ui.CatalogImport;
import net.refractions.udig.catalog.internal.ui.ResourceSelectionPage;
import net.refractions.udig.catalog.ui.workflow.DataSourceSelectionState;
import net.refractions.udig.catalog.ui.workflow.ResourceSelectionState;
import net.refractions.udig.catalog.ui.workflow.Workflow;
import net.refractions.udig.catalog.ui.workflow.WorkflowWizard;
import net.refractions.udig.catalog.ui.workflow.WorkflowWizardPage;
import net.refractions.udig.catalog.ui.workflow.Workflow.State;
import net.refractions.udig.project.ui.ApplicationGIS;

import org.eclipse.core.runtime.IProgressMonitor;

public class MapImport extends CatalogImport {

    private int layerPosition = -1;

    public void setLayerPosition( int layerPosition ) {
        this.layerPosition = layerPosition;
    }

    @Override
    protected Workflow createWorkflow() {
        DataSourceSelectionState dsState = new DataSourceSelectionState(true);
        ResourceSelectionState rsState = new ResourceSelectionState();

        Workflow workflow = new Workflow(new Workflow.State[]{dsState, rsState});
        return workflow;
    }

    @Override
    protected java.util.Map<Class< ? extends State>, WorkflowWizardPage> createPageMapping() {
        java.util.Map<Class< ? extends State>, WorkflowWizardPage> map = super.createPageMapping();
        map.put(ResourceSelectionState.class, new ResourceSelectionPage(Messages.MapImport_selectLayers)); 

        return map;
    }

    @Override
    protected WorkflowWizard createWorkflowWizard( Workflow workflow,
            java.util.Map<Class< ? extends State>, WorkflowWizardPage> map ) {
        return new MapImportWizard(workflow, map);
    }

    protected class MapImportWizard extends CatalogImport.CatalogImportWizard {

        public MapImportWizard( Workflow workflow,
                java.util.Map<Class< ? extends State>, WorkflowWizardPage> map ) {
            super(workflow, map);
            setWindowTitle("Add Data");
        } 
        
        @Override
        public boolean canFinish() {
            return super.canFinish();
        }
        
        @Override
        protected boolean performFinish( IProgressMonitor monitor ) {
            String name = Messages.MapImport_createMap; 
            monitor.beginTask(name, IProgressMonitor.UNKNOWN);
            monitor.setTaskName(name);
            boolean superFinished = super.performFinish(monitor);
            if (!superFinished)
                return superFinished;

            ResourceSelectionState state = getWorkflow().getState(ResourceSelectionState.class);
            java.util.Map<IGeoResource, IService> resourceMap = state.getResources();

            if (resourceMap == null || resourceMap.isEmpty())
                return false;

            // add the resources to the map
            List<IGeoResource> resourceList = new ArrayList<IGeoResource>(resourceMap.keySet());

            monitor.setTaskName(Messages.MapImport_addingLayersTask); 
            ApplicationGIS.addLayersToMap(ApplicationGIS.getActiveMap(), resourceList, layerPosition, null, true);

            return true;
        }

        @Override
        protected boolean isShowCatalogView() {
            return false;
        }
    }

}
