/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.command.navigation;

import java.util.Iterator;
import java.util.List;

import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.command.NavCommand;
import net.refractions.udig.project.command.PostDeterminedEffectCommand;
import net.refractions.udig.project.command.UndoableComposite;
import net.refractions.udig.project.internal.render.ViewportModel;
import net.refractions.udig.project.internal.render.impl.ViewportModelImpl;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;

/**
 * TODO Purpose of net.refractions.udig.project.internal.command.navigation
 * <p>
 * </p>
 * 
 * @author Jesse
 * @since 1.0.0
 */
public class NavComposite extends UndoableComposite implements NavCommand {

    ViewportModel model;

    /**
     * Creates a new instance of NavComposite
     * 
     * @param navCommands an ordered list of Nav commands
     */
    public NavComposite( List navCommands ) {
        super(navCommands);
    }

    /**
     * @see net.refractions.udig.project.internal.command.navigation.NavCommand#setViewportModel(net.refractions.udig.project.ViewportModelControl)
     */
    public void setViewportModel( ViewportModel model ) {
        this.model = model;
        for( Iterator iter = commands.iterator(); iter.hasNext(); ) {
            NavCommand command = (NavCommand) iter.next();
            command.setViewportModel(model);
        }
    }

    public void run( IProgressMonitor monitor ) throws Exception {
        execute(monitor);
    }
    
    public boolean execute( IProgressMonitor monitor ) throws Exception {
        monitor.beginTask(getName(), 12*commands.size());
        monitor.worked(2);

        ViewportModelImpl hack = (ViewportModelImpl) model;
        hack.setFiringEvents(false);
        
        boolean changedState=false;
        for (Iterator<? extends MapCommand> iter=commands.iterator(); iter.hasNext(); ) {
            NavCommand command = (NavCommand) iter.next();

            if (!iter.hasNext())
                hack.setFiringEvents(true);
            command.setMap(getMap());
            SubProgressMonitor subProgressMonitor = new SubProgressMonitor(monitor, 10);
            if( command instanceof PostDeterminedEffectCommand){
                boolean change=((PostDeterminedEffectCommand) command).execute(subProgressMonitor);
                changedState=changedState||change;
            }else{
                command.run(subProgressMonitor);
                changedState=true;
            }
            subProgressMonitor.done();
        }
        monitor.done();

        return changedState;
    }

}
