package net.refractions.udig.style.sld.editor;

import java.util.Iterator;

import net.refractions.udig.core.AdapterUtil;
import net.refractions.udig.core.internal.ExtensionPointProcessor;
import net.refractions.udig.core.internal.ExtensionPointUtil;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.style.IStyleConfigurator;
import net.refractions.udig.style.sld.SLDPlugin;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;

/**
 * Open the style editor dialog and add its pages
 */
public class OpenStyleEditorAction extends Action implements IWorkbenchWindowActionDelegate, IViewActionDelegate {
    public static final String ATT_ID = "id"; //$NON-NLS-1$
    public static final String ATT_CLASS = "class"; //$NON-NLS-1$
    public static final String ATT_LABEL = "label"; //$NON-NLS-1$
    public static final String ATT_REQUIRES = "requires"; //$NON-NLS-1$
    public static final String STYLE_ID = "net.refractions.udig.style.sld"; //$NON-NLS-1$

    private Layer selectedLayer;
    
    private Plugin plugin;
    
    /**
     * The workbench window; or <code>null</code> if this
     * action has been <code>dispose</code>d.
     */
    private IWorkbenchWindow workbenchWindow;

    /**
     * Create a new <code>OpenPreferenceAction</code>
     * This default constructor allows the the action to be called from the welcome page.
     */
    public OpenStyleEditorAction() {
        this(PlatformUI.getWorkbench().getActiveWorkbenchWindow());
    }

    /**
     * Create a new <code>OpenPreferenceAction</code> and initialize it 
     * from the given resource bundle.
     * @param window
     */
    public OpenStyleEditorAction(IWorkbenchWindow window) {
        super();  
        if (window == null) {
            throw new IllegalArgumentException();
        }
        this.workbenchWindow = window;
        setActionDefinitionId("net.refractions.style.sld.editor"); //$NON-NLS-1$
        
		//setToolTipText(WorkbenchMessages.OpenPreferences_toolTip); 
		//window.getWorkbench().getHelpSystem().setHelp(this,
		//       IWorkbenchHelpContextIds.OPEN_PREFERENCES_ACTION);
    }

    public void run( IAction action ) {
        if (workbenchWindow == null) {
            return; // action has been disposed
        }
        
        Shell shell = workbenchWindow.getShell();
        //the page to select by default
        String pageId = "simple";  //$NON-NLS-1$
        //the filter to apply, if defined
        //String[] displayedIds = null; 
        
        final EditorPageManager[] manager = new EditorPageManager[] {new EditorPageManager('.')};
        
        ExtensionPointProcessor extProcPage = new ExtensionPointProcessor() {
            public void process( IExtension extension, IConfigurationElement element ) throws Exception {
                EditorNode node = null;
                    
                boolean labelMissing = element.getAttribute(ATT_LABEL) == null;
                String id = element.getAttribute(ATT_ID);
                String requires = element.getAttribute(ATT_REQUIRES);
                boolean classMissing = manager[0].getClassValue(element, ATT_CLASS) == null;

                if (!(labelMissing || id == null || classMissing)) {
                    node = new EditorNode(id, element);
                }
                if (node == null)
                    return;

                if (requires != null && !meetsRequirement(id, element, node)) {
                    return;
                }
                
                manager[0].registerNode(node);
                String category = node.getCategory();
                if (category == null) {
                    manager[0].addToRoot(node);
                } else {
                    EditorNode parent = null;
                    for (Iterator j = manager[0].getElements(EditorPageManager.POST_ORDER).iterator(); j.hasNext();) {
                        EditorNode pNode = (EditorNode) j.next();
                        if (category.equals(pNode.getId())) {
                            parent = pNode;
                            break;
                        }
                    }
                    if (parent == null) {
                        //TODO: log error
                        manager[0].addToRoot(node);
                    } else {
                        parent.add(node);
                    }
                }
            }
        };
        ExtensionPointUtil.process(plugin, StyleEditorPage.XPID, extProcPage);

        ExtensionPointProcessor extProcConfigurator = new ExtensionPointProcessor() {
            public void process( IExtension extension, IConfigurationElement element ) throws Exception {
                EditorNode node = null;
                    
                String id = element.getAttribute(ATT_ID);
                String label = element.getAttribute(ATT_LABEL);
                String requires = element.getAttribute(ATT_REQUIRES);
                boolean classMissing = manager[0].getClassValue(element, ATT_CLASS) == null;
                if (label == null) {
                    SLDPlugin.trace("StyleConfigurator extension point attribute 'label' not specified -- instance ignored: " + element.getClass(), null); //$NON-NLS-1$
                    return;
                }
                if (id == null) {
                    //if the optional id was not defined, skip
                    SLDPlugin.trace("StyleConfigurator extension point attribute 'id' not specified -- skipped", null); //$NON-NLS-1$
                    return;
                }
                if (classMissing) {
                    SLDPlugin.trace("StyleConfigurator extension point class could not be found: " + element.getClass(), null); //$NON-NLS-1$
                    return;
                }
                if (manager[0].hasNode(id)) {
                    SLDPlugin.trace("Duplicate id found -- skipping '" + id + "' with label '" + label + "'", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                    return;
                }
                
                node = new EditorNode(id, element);

                if (node == null) {
                    SLDPlugin.log("EditorNode creation failed in OpenStyleEditorAction", null); //$NON-NLS-1$
                    return;
                }

                if (requires != null && !meetsRequirement(id, element, node) ) {
                    return;
                }
                
                manager[0].registerNode(node);
                String category = node.getCategory();
                if (category == null) {
                    manager[0].addToRoot(node);
                } else {
                    EditorNode parent = null;
                    for (Iterator j = manager[0].getElements(EditorPageManager.POST_ORDER).iterator(); j.hasNext();) {
                        EditorNode pNode = (EditorNode) j.next();
                        if (category.equals(pNode.getId())) {
                            parent = pNode;
                            break;
                        }
                    }
                    if (parent == null) {
                        //TODO: log error
                        manager[0].addToRoot(node);
                    } else {
                        parent.add(node);
                    }
                }
            }

        };
        ExtensionPointUtil.process(plugin, IStyleConfigurator.XPID, extProcConfigurator);

        StyleEditorDialog dialog = StyleEditorDialog.createDialogOn(shell, pageId, selectedLayer, manager[0]);
        //do filtering
        //if (displayedIds != null)
        //    dialog.showOnly(displayedIds);
        //open the dialog
        dialog.open();
    }
    
    boolean meetsRequirement( String id, IConfigurationElement element, EditorNode node ) {
        String requires = element.getAttribute(ATT_REQUIRES);
        try {
            Object classInstance = EditorNode.createExtension(element, EditorNode.ATT_CLASS);
            // first try creating required class using extension classloading 
            // if this fails use the same classloader as the configurator
            try{
                Object requiredClass = element.createExecutableExtension(ATT_REQUIRES);
                if ( !AdapterUtil.instance.canAdaptTo(requires, selectedLayer, requiredClass.getClass().getClassLoader())) {
                    SLDPlugin.trace("skipped "+id, null); //$NON-NLS-1$
                    return false;
                }
                
            }catch( Exception ce ){
                // Failed trying to recover by using the configurators class's class loader
                if (!AdapterUtil.instance.canAdaptTo(requires, selectedLayer, classInstance.getClass().getClassLoader())) {
                    SLDPlugin.trace("skipped "+id, null); //$NON-NLS-1$
                    return false;
                }
            }
        } catch (Exception e) {
            SLDPlugin.log("extProcConfigurator skipped " //$NON-NLS-1$
                          + id + " (couldn't find " //$NON-NLS-1$
                          + requires + ")", null); //$NON-NLS-1$
            return  false;
        }
        return true;
    }

    
    public void selectionChanged( IAction action, ISelection selection ) {
        if (selection.isEmpty() || !(selection instanceof IStructuredSelection)) 
            return;
        
        IStructuredSelection sselection = (IStructuredSelection)selection;
        if (sselection.getFirstElement() instanceof Layer) {
            selectedLayer = (Layer) sselection.getFirstElement();
        }
    }

    public void dispose() {
    	plugin = null;
    	workbenchWindow = null;
    }

    public void init( IWorkbenchWindow window ) {
    }

    public void init( IViewPart view ) {
    }

}
