package net.refractions.udig.style.sld.editor;

import java.awt.Color;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import net.refractions.udig.style.internal.StyleLayer;
import net.refractions.udig.style.sld.ImageConstants;
import net.refractions.udig.style.sld.Images;
import net.refractions.udig.style.sld.SLDContent;
import net.refractions.udig.style.sld.SLDPlugin;
import net.refractions.udig.style.sld.editor.internal.BrewerPaletteLabelProvider;
import net.refractions.udig.style.sld.editor.internal.StyleTreeContentProvider;
import net.refractions.udig.style.sld.editor.internal.StyleTreeLabelProvider;
import net.refractions.udig.style.sld.editor.internal.StyleTreeSorter;
import net.refractions.udig.style.sld.internal.Messages;
import net.refractions.udig.ui.DecoratorOverlayIcon;
import net.refractions.udig.ui.ErrorManager;
import net.refractions.udig.ui.PlatformGIS;
import net.refractions.udig.ui.graphics.SLDs;
import net.refractions.udig.ui.graphics.TableSettings;
import net.refractions.udig.ui.graphics.TableUtils;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColorCellEditor;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.geotools.brewer.color.BrewerPalette;
import org.geotools.brewer.color.ColorBrewer;
import org.geotools.brewer.color.PaletteSuitability;
import org.geotools.brewer.color.SampleScheme;
import org.geotools.brewer.color.StyleGenerator;
import org.geotools.data.FeatureSource;
import org.geotools.event.GTComponent;
import org.geotools.event.GTEvent;
import org.geotools.factory.CommonFactoryFinder;
import org.geotools.factory.GeoTools;
import org.geotools.feature.AttributeType;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureType;
import org.geotools.feature.GeometryAttributeType;
import org.geotools.feature.type.NumericAttributeType;
import org.geotools.feature.visitor.UniqueVisitor;
import org.geotools.filter.IllegalFilterException;
import org.geotools.filter.function.ClassificationFunction;
import org.geotools.filter.function.Classifier;
import org.geotools.filter.function.EqualIntervalFunction;
import org.geotools.filter.function.QuantileFunction;
import org.geotools.filter.function.StandardDeviationFunction;
import org.geotools.filter.function.UniqueIntervalFunction;
import org.geotools.styling.FeatureTypeStyle;
import org.geotools.styling.Fill;
import org.geotools.styling.LineSymbolizer;
import org.geotools.styling.PointSymbolizer;
import org.geotools.styling.PolygonSymbolizer;
import org.geotools.styling.Rule;
import org.geotools.styling.Stroke;
import org.geotools.styling.Style;
import org.geotools.styling.StyleBuilder;
import org.geotools.styling.StyledLayerDescriptor;
import org.geotools.styling.Symbolizer;
import org.geotools.util.NullProgressListener;
import org.opengis.filter.FilterFactory;
import org.opengis.filter.expression.Expression;
import org.opengis.util.ProgressListener;

public class StyleThemePage extends StyleEditorPage {

    public static final int COMBO_ATTRIBUTES = 1;
    public static final int COMBO_CLASSES = 2;
    public static final int COMBO_BREAKTYPE = 3;
    public static final int COMBO_NORMALIZE = 4;
    public static final int COMBO_ELSE = 5;
    public static final int COMBO_PALETTES = 6;
    public static final int COMBO_OPACITY = 7;

    public static final int LABEL_PALETTES = 10;
    public static final int LABEL_SEPARATOR_BOTTOM = 12;
    public static final int LABEL_STATUSBAR = 13;
    
    public static final int LABEL_ICON_COLORBLIND = 14;
    public static final int LABEL_ICON_PHOTOCOPY = 15;
    public static final int LABEL_ICON_PROJECTOR = 16;
    public static final int LABEL_ICON_LCD = 17;
    public static final int LABEL_ICON_CRT = 18;
    public static final int LABEL_ICON_PRINT = 19;
    
    public static final int BUTTON_COLORBLIND = 20;
    public static final int BUTTON_PHOTOCOPY = 21;
    public static final int BUTTON_PROJECTOR = 22;
    public static final int BUTTON_LCD = 23;
    public static final int BUTTON_CRT = 24;
    public static final int BUTTON_PRINT = 25;
    public static final int BUTTON_REMOVE = 26;
    
    public static final int COMPOSITE_PARENT = 30;
    public static final int COMPOSITE_TOP = 31;
    public static final int COMPOSITE_MIDDLE = 32;
    public static final int COMPOSITE_BOTTOM = 33;
    public static final int COMPOSITE_BOTTOM_LEFT = 34;
    public static final int COMPOSITE_BOTTOM_RIGHT = 35;
    
    private HashMap<Integer,Control> pageControls = new HashMap<Integer,Control>();
    private List<String> numericAttr = new ArrayList<String>();
    double[] opacity;    

    HashMap<Integer, String> controlNames = new HashMap<Integer, String>();
    HashMap<Integer, Integer> viewerQuality = new HashMap<Integer, Integer>();
    HashMap<String, Integer> uniqueCounts = new HashMap<String, Integer>();
    
    TableViewer paletteTable;
    ColorBrewer brewer;
    BrewerPalette customPalette = null;
    boolean reverseColours = false;
    StyleGenerator sg;
    TreeViewer treeViewer;
    AttributeType selectedAttributeType;
    AttributeType normalize;
    ClassificationFunction classifierFunc;
    Classifier classifier;
    TableSettings tableSettings;
    boolean elseSelection = true;
    
    FilterFactory ff;
    FeatureCollection collection;
    FeatureSource source;
    StyleBuilder sb;

    
    public StyleThemePage() {
        //create factories
        ff = CommonFactoryFinder.getFilterFactory(GeoTools.getDefaultHints());
        sb = new StyleBuilder(CommonFactoryFinder.getStyleFactory(GeoTools.getDefaultHints()));
        
        //suitability icons
        controlNames.put(LABEL_ICON_COLORBLIND, Messages.StyleEditor_theme_suitability_colour); 
        controlNames.put(LABEL_ICON_CRT, Messages.StyleEditor_theme_suitability_crt); 
        controlNames.put(LABEL_ICON_LCD, Messages.StyleEditor_theme_suitability_lcd); 
        controlNames.put(LABEL_ICON_PHOTOCOPY, Messages.StyleEditor_theme_suitability_pcopy); 
        controlNames.put(LABEL_ICON_PRINT, Messages.StyleEditor_theme_suitability_print); 
        controlNames.put(LABEL_ICON_PROJECTOR, Messages.StyleEditor_theme_suitability_proj); 

        //suitability toggle buttons
        controlNames.put(BUTTON_COLORBLIND, Messages.StyleEditor_theme_suitability_colour); 
        controlNames.put(BUTTON_CRT, Messages.StyleEditor_theme_suitability_crt); 
        controlNames.put(BUTTON_LCD, Messages.StyleEditor_theme_suitability_lcd); 
        controlNames.put(BUTTON_PHOTOCOPY, Messages.StyleEditor_theme_suitability_pcopy); 
        controlNames.put(BUTTON_PRINT, Messages.StyleEditor_theme_suitability_print); 
        controlNames.put(BUTTON_PROJECTOR, Messages.StyleEditor_theme_suitability_proj); 

        //suitability icon - palette suitability lookup
        viewerQuality.put(PaletteSuitability.VIEWER_COLORBLIND, LABEL_ICON_COLORBLIND);
        viewerQuality.put(PaletteSuitability.VIEWER_CRT, LABEL_ICON_CRT);
        viewerQuality.put(PaletteSuitability.VIEWER_LCD, LABEL_ICON_LCD);
        viewerQuality.put(PaletteSuitability.VIEWER_PHOTOCOPY, LABEL_ICON_PHOTOCOPY);
        viewerQuality.put(PaletteSuitability.VIEWER_PRINT, LABEL_ICON_PRINT);
        viewerQuality.put(PaletteSuitability.VIEWER_PROJECTOR, LABEL_ICON_PROJECTOR);
    }
    
    public Button getButton(int widgetID) {
        return (Button) getControl(widgetID);
    }
    
    public Combo getCombo(int widgetID) {
        return (Combo) getControl(widgetID);
    }

    public Composite getComposite(int widgetID) {
        return (Composite) getControl(widgetID);
    }
    
    public Label getLabel(int widgetID) {
        return (Label) getControl(widgetID);
    }

    public GridData getLayoutData(int widgetID) {
        Object widget = getControl(widgetID);
        Object layout = null;
        if (widget instanceof Composite) {
            layout = ((Composite) widget).getLayoutData();
        }
        if (layout instanceof GridData) return (GridData) layout;
        else return new GridData(); //return empty, or throw exception (null)?
    }
    
    public Object getControl(int widgetID) {
        return pageControls.get(widgetID);
    }
    
    private boolean isNumber(String attributeType) {
        return numericAttr.contains(attributeType);
    }
    
    private boolean isNumber(AttributeType attributeType) {
        if (attributeType instanceof NumericAttributeType) return true;
        else return false;
    }

    @Override
    public boolean isValid() {
        return true;
    }

    public boolean okToLeave() {
        return true;
    }

    @Override
    public boolean performCancel() {
        return true;
    }

    public boolean performOk() {
        return true;
    }

    @Override
    public void dispose() {
        super.dispose();
        //collection.purge();
        collection = null;
        source = null;
    }

    @Override
    public String getErrorMessage() {
        return null;
    }

    public ColorBrewer getBrewer() {
        if (brewer == null) {
            createBrewer();
        }
        return brewer;
    }
    
    private void createToggleButton(Composite parent, int buttonId, Image image) {
        Button toggleButton = new Button(parent, SWT.TOGGLE);
        GridData gridData = new GridData(SWT.NONE, SWT.NONE, false, false);
        toggleButton.setLayoutData(gridData);
        toggleButton.setImage(image);
        toggleButton.setSelection(false);
        toggleButton.addSelectionListener(new SuitabilityToggleListener(buttonId));
        pageControls.put(buttonId, toggleButton);
        updateToggleTooltip(buttonId);
    }
    
    private void updateToggleTooltip(int buttonId) {
        Button toggle = getButton(buttonId);
        String tooltip;
        if (toggle.getSelection()) {
            tooltip = Messages.StyleEditor_theme_suitability_show; 
        } else {
            tooltip = Messages.StyleEditor_theme_suitability_hide; 
        }
        toggle.setToolTipText(tooltip+" "+controlNames.get(buttonId)); //$NON-NLS-1$
    }
    
    private void createSuitabilityIcon(Composite parent, int id, String tooltip, Image image) {
        Label suitabilityIcon = new Label(parent, SWT.NONE);
        GridData gridData = new GridData(SWT.NONE, SWT.NONE, false, false);
        suitabilityIcon.setLayoutData(gridData);
        suitabilityIcon.setImage(image);
        suitabilityIcon.setToolTipText(tooltip);
        pageControls.put(id, suitabilityIcon);
    }
    
    @Override
    public void createPageContent( Composite parent ) {
        //create the featureCollection
        IProgressMonitor monitor = new NullProgressMonitor();
        try {
            source = getSelectedLayer().getResource(FeatureSource.class, monitor); //monitor
            if (source == null) return;
            collection = source.getFeatures();
        } catch (IOException e) {
            SLDPlugin.log("StyleThemePage.createPageContent() failed to create FeatureCollection", e); //$NON-NLS-1$
            return;
        }

        //create the content
        pageControls.put(COMPOSITE_PARENT, parent);
        Font font = getShell().getFont();
        
        Composite compTop = new Composite(parent, SWT.NONE);
        compTop.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        GridLayout layout = new GridLayout(5, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        compTop.setLayout(layout);
        compTop.setFont(font);
        pageControls.put(COMPOSITE_TOP, compTop);
        
        Label separator = new Label(parent, SWT.HORIZONTAL | SWT.SEPARATOR);
        GridData gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
        separator.setLayoutData(gridData);
        separator.setFont(font);
        
        Composite compMiddle = new Composite(parent, SWT.NONE);
        compMiddle.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        layout = new GridLayout(1, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        compMiddle.setLayout(layout);
        compMiddle.setFont(font);
        pageControls.put(COMPOSITE_MIDDLE, compMiddle);

        separator = new Label(parent, SWT.HORIZONTAL | SWT.SEPARATOR);
        gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
        separator.setLayoutData(gridData);
        separator.setVisible(false);
        separator.setFont(font);
        pageControls.put(LABEL_SEPARATOR_BOTTOM, separator);

        Composite compBottom = new Composite(parent, SWT.NONE);
        gridData = new GridData(SWT.FILL, SWT.FILL, true, false);
        gridData.heightHint = 0;
        compBottom.setLayoutData(gridData);
        layout = new GridLayout(2, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        compBottom.setLayout(layout);
        compBottom.setVisible(false);
        compBottom.setFont(font);
        pageControls.put(COMPOSITE_BOTTOM, compBottom);

        //Labels
        Label attributesLabel = new Label(compTop, SWT.NONE);
        attributesLabel.setFont(font);
        attributesLabel.setText(Messages.StyleEditor_theme_attribute); 
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        attributesLabel.setLayoutData(gridData);

        Label classesLabel = new Label(compTop, SWT.NONE);
        classesLabel.setFont(font);
        classesLabel.setText(Messages.StyleEditor_theme_classes); 
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        classesLabel.setLayoutData(gridData);

        Label breaksLabel = new Label(compTop, SWT.NONE);
        breaksLabel.setFont(font);
        breaksLabel.setText(Messages.StyleEditor_theme_break); 
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        breaksLabel.setLayoutData(gridData);

        Label normalizeLabel = new Label(compTop, SWT.LEFT);
        normalizeLabel.setFont(font);
        normalizeLabel.setText(Messages.StyleEditor_theme_normalize); 
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        normalizeLabel.setLayoutData(gridData);

        Label elseLabel = new Label(compTop, SWT.LEFT);
        elseLabel.setFont(font);
        elseLabel.setText(Messages.StyleEditor_theme_else); 
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        elseLabel.setLayoutData(gridData);

        //Combos
        Combo attributeCombo = new Combo(compTop, SWT.BORDER | SWT.READ_ONLY);
        pageControls.put(COMBO_ATTRIBUTES, attributeCombo);
        attributeCombo.setLayout(new GridLayout());
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        attributeCombo.setLayoutData(gridData);
        //populate the comboBox
        StyleLayer selectedLayer = getSelectedLayer();
        if (selectedLayer != null) {
            FeatureType featureType = selectedLayer.getSchema();
            
            if (featureType != null) {
                for (int i = 0; i < featureType.getAttributeCount(); i++) {
                    AttributeType attributeType = featureType.getAttributeType(i);
                    if (!(attributeType instanceof GeometryAttributeType)) { //don't include the geometry
                        attributeCombo.add(attributeType.getName());
                        if (isNumber(attributeType)) {
                            numericAttr.add(attributeType.getName());
                        }
                    }
                }
                //select the first numeric attribute that isn't ID
                int index = -1;
                if (numericAttr.size() > 1) {
                    if (numericAttr.get(0).equalsIgnoreCase("id")) { //$NON-NLS-1$
                        index = attributeCombo.indexOf(numericAttr.get(1));
                    } else {
                        index = attributeCombo.indexOf(numericAttr.get(0));
                    }
                } else if (numericAttr.size() == 1) {
                    index = attributeCombo.indexOf(numericAttr.get(0));
                }
                if (index > -1) attributeCombo.select(index);
                if (attributeCombo.getSelectionIndex() == -1) {
                    //we couldn't find a desirable attribute, just grab the first one 
                    if (featureType.getAttributeCount() > 0) {
                        attributeCombo.select(0);
                    }
                }
            }
        }
        attributeCombo.setVisibleItemCount(16);
        attributeCombo.addSelectionListener(new AttributeComboListener());
        
        Combo classesCombo = new Combo(compTop, SWT.BORDER | SWT.READ_ONLY);
        pageControls.put(COMBO_CLASSES, classesCombo);
        classesCombo.setLayout(new GridLayout(1, false));
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        classesCombo.setLayoutData(gridData);
        for (int i = 2; i < 13; i++) {
            Integer j = (Integer) i;
            classesCombo.add(j.toString());
        }
        classesCombo.select(3); //default is 5 classes
        classesCombo.setVisibleItemCount(16);
        classesCombo.addSelectionListener(new ClassesComboListener());

        Combo breaksCombo = new Combo(compTop, SWT.BORDER | SWT.READ_ONLY);
        pageControls.put(COMBO_BREAKTYPE, breaksCombo);
        breaksCombo.setLayout(new GridLayout(1, false));
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        breaksCombo.setLayoutData(gridData);
        updateBreaks();
        breaksCombo.addSelectionListener(new SimpleComboListener());

        Combo normalizeCombo = new Combo(compTop, SWT.BORDER | SWT.READ_ONLY);
        pageControls.put(COMBO_NORMALIZE, normalizeCombo);
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        normalizeCombo.setLayoutData(gridData);
        updateNormalize();
        normalizeCombo.setToolTipText(Messages.StyleEditor_theme_normalize_tip); 
        normalizeCombo.setVisibleItemCount(10);
        normalizeCombo.addSelectionListener(new SimpleComboListener());

        Combo elseCombo = new Combo(compTop, SWT.BORDER | SWT.READ_ONLY);
        pageControls.put(COMBO_ELSE, elseCombo);
        elseCombo.setLayout(new GridLayout(1, false));
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
        elseCombo.setLayoutData(gridData);
        elseCombo.add(Messages.StyleEditor_theme_else_hide); 
        elseCombo.add(Messages.StyleEditor_theme_else_min); 
        elseCombo.add(Messages.StyleEditor_theme_else_max); 
        elseCombo.select(0);
        elseCombo.setToolTipText(Messages.StyleEditor_theme_else_tip); 
        elseCombo.addSelectionListener(new ElseComboListener());
        
        Composite compMiddle2 = new Composite(compMiddle, SWT.NONE);
        compMiddle2.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
        layout = new GridLayout(2, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        compMiddle2.setLayout(layout);

        Label palettesLabel = new Label(compMiddle2, SWT.LEFT);
        palettesLabel.setFont(font);
        palettesLabel.setText(Messages.StyleEditor_theme_palette); 
        gridData = new GridData(SWT.LEFT, SWT.BOTTOM, true, false);
        palettesLabel.setLayoutData(gridData);
        
        Composite compMiddle3 = new Composite(compMiddle2, SWT.NONE);
        compMiddle3.setLayoutData(new GridData(SWT.RIGHT, SWT.NONE, true, false));
        layout = new GridLayout(3, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        compMiddle3.setLayout(layout);

        Label paletteFilterLabel = new Label(compMiddle3, SWT.LEFT);
        paletteFilterLabel.setFont(font);
        paletteFilterLabel.setText(Messages.StyleEditor_theme_show); 
        gridData = new GridData(SWT.LEFT, SWT.FILL, false, true);
        gridData.verticalSpan = 2;
        paletteFilterLabel.setLayoutData(gridData);
        
        Combo paletteFilter = new Combo(compMiddle3, SWT.BORDER | SWT.READ_ONLY);
        pageControls.put(COMBO_PALETTES, paletteFilter);
        paletteFilter.add(Messages.StyleEditor_theme_palette_all, 0); 
        paletteFilter.add(Messages.StyleEditor_theme_palette_num, 1); 
        paletteFilter.add(Messages.StyleEditor_theme_palette_seq, 2); 
        paletteFilter.add(Messages.StyleEditor_theme_palette_div, 3); 
        paletteFilter.add(Messages.StyleEditor_theme_palette_cat, 4); 
        paletteFilter.select(0);
        paletteFilter.setToolTipText(Messages.StyleEditor_theme_palette_tip); 
        paletteFilter.addSelectionListener(new SelectionListener() {

            public void widgetSelected( SelectionEvent e ) {
                paletteTable.refresh();
            }

            public void widgetDefaultSelected( SelectionEvent e ) {
                widgetSelected(e);
            }
            
        });

        Composite compMiddle4 = new Composite(compMiddle3, SWT.NONE);
        compMiddle4.setLayoutData(new GridData(SWT.RIGHT, SWT.NONE, true, false));
        layout = new GridLayout(6, true);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        layout.horizontalSpacing = 0;
        compMiddle4.setLayout(layout);
        
        //toggle buttons
        createToggleButton(compMiddle4, BUTTON_COLORBLIND, Images.getDescriptor(ImageConstants.COLORBLIND_ICON).createImage());
        createToggleButton(compMiddle4, BUTTON_CRT, Images.getDescriptor(ImageConstants.CRT_ICON).createImage());
        createToggleButton(compMiddle4, BUTTON_PROJECTOR, Images.getDescriptor(ImageConstants.PROJECTOR_ICON).createImage());
        createToggleButton(compMiddle4, BUTTON_LCD, Images.getDescriptor(ImageConstants.LAPTOP_ICON).createImage());
        createToggleButton(compMiddle4, BUTTON_PRINT, Images.getDescriptor(ImageConstants.PRINTER_ICON).createImage());
        createToggleButton(compMiddle4, BUTTON_PHOTOCOPY, Images.getDescriptor(ImageConstants.PHOTOCOPY_ICON).createImage());
        
        //list of matching palettes
        paletteTable = new TableViewer(new Table(compMiddle, SWT.BORDER | SWT.V_SCROLL | SWT.FULL_SELECTION));
        TableLayout tableLayout = new TableLayout();
        tableLayout.addColumnData(new ColumnWeightData(1, 20, false));
        TableColumn firstColumn = new TableColumn(paletteTable.getTable(), SWT.LEFT);
        firstColumn.setAlignment(SWT.LEFT);
        paletteTable.getTable().setLayout(tableLayout);
        gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
        gridData.horizontalSpan = 2;
        gridData.heightHint = 150;
        gridData.widthHint = 175;
        paletteTable.getControl().setLayoutData(gridData);

        paletteTable.setLabelProvider(new BrewerPaletteLabelProvider());
        paletteTable.setContentProvider(new IStructuredContentProvider() {
            
            public Object[] getElements(Object inputElement) {
              if (inputElement instanceof ArrayList) {
                  ArrayList list = (ArrayList) inputElement;
                  return list.toArray();
              } else 
                if (inputElement instanceof ColorBrewer) {
                    ColorBrewer brewer = (ColorBrewer) inputElement;
                    int selection = getCombo(COMBO_PALETTES).getSelectionIndex();
                    if (selection == 0) //All
                        return brewer.getPalettes(ColorBrewer.ALL); 
                    else if (selection == 1) //Numerical
                        return brewer.getPalettes(ColorBrewer.SUITABLE_RANGED);
                    else if (selection == 2) //Sequential
                        return brewer.getPalettes(ColorBrewer.SEQUENTIAL);
                    else if (selection == 3) //Diverging
                        return brewer.getPalettes(ColorBrewer.DIVERGING);
                    else if (selection == 4) //Categorical
                        return brewer.getPalettes(ColorBrewer.SUITABLE_UNIQUE);
                    else
                        return brewer.getPalettes();
                } else {
                    return new Object[0];
                }
            } 

            public void dispose() {
            }

            public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
            }
        });

        paletteTable.addFilter(new ViewerFilter() {

            @Override
            public boolean select( Viewer viewer, Object parentElement, Object element ) {
                if (element instanceof BrewerPalette) {
                    BrewerPalette pal = (BrewerPalette) element;
                    int numClasses = new Integer(getCombo(COMBO_CLASSES).getText()).intValue();
                    if (pal.getMaxColors() < numClasses) {
                        return false;
                    }
                    if (getButton(BUTTON_COLORBLIND).getSelection()) {
                        if (pal.getPaletteSuitability().getSuitability(numClasses, PaletteSuitability.VIEWER_COLORBLIND) != PaletteSuitability.QUALITY_GOOD)
                            return false;
                    }
                    if (getButton(BUTTON_CRT).getSelection()) {
                        if (pal.getPaletteSuitability().getSuitability(numClasses, PaletteSuitability.VIEWER_CRT) != PaletteSuitability.QUALITY_GOOD)
                            return false;
                    }
                    if (getButton(BUTTON_LCD).getSelection()) {
                        if (pal.getPaletteSuitability().getSuitability(numClasses, PaletteSuitability.VIEWER_LCD) != PaletteSuitability.QUALITY_GOOD)
                            return false;
                    }
                    if (getButton(BUTTON_PHOTOCOPY).getSelection()) {
                        if (pal.getPaletteSuitability().getSuitability(numClasses, PaletteSuitability.VIEWER_PHOTOCOPY) != PaletteSuitability.QUALITY_GOOD)
                            return false;
                    }
                    if (getButton(BUTTON_PRINT).getSelection()) {
                        if (pal.getPaletteSuitability().getSuitability(numClasses, PaletteSuitability.VIEWER_PRINT) != PaletteSuitability.QUALITY_GOOD)
                            return false;
                    }
                    if (getButton(BUTTON_PROJECTOR).getSelection()) {
                        if (pal.getPaletteSuitability().getSuitability(numClasses, PaletteSuitability.VIEWER_PROJECTOR) != PaletteSuitability.QUALITY_GOOD)
                            return false;
                    }
                }
                return true;
            }
            
        });
        paletteTable.setSorter(new ViewerSorter() {

            @Override
            public int compare( Viewer viewer, Object e1, Object e2 ) {
                if (e1 instanceof BrewerPalette && e2 instanceof BrewerPalette) {
                    BrewerPalette p1 = (BrewerPalette) e1;
                    BrewerPalette p2 = (BrewerPalette) e2;
                    //alphabetical by name
                    return p1.getName().compareTo(p2.getName());
                    //TODO: alternatives (colour hue?)
                } else return super.compare(viewer, e1, e2);
            }
            
        });
        
        paletteTable.setInput(getBrewer());
        paletteTable.addSelectionChangedListener(new PalettesListener());
        
        //Bottom composite
        Composite compBottomLeft = new Composite(compBottom, SWT.NONE);
        compBottomLeft.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        layout = new GridLayout(1, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        compBottomLeft.setLayout(layout);
        pageControls.put(COMPOSITE_BOTTOM_LEFT, compBottomLeft);
        
        Composite compBottomLeftButtons = new Composite(compBottomLeft, SWT.NONE);
        compBottomLeftButtons.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
        layout = new GridLayout(4, true);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        compBottomLeftButtons.setLayout(layout);
        
        Label opacityLabel = new Label(compBottomLeftButtons, SWT.NONE);
        opacityLabel.setFont(font);
        opacityLabel.setText(Messages.StyleEditor_theme_opacity); 
        gridData = new GridData(SWT.LEFT, SWT.DEFAULT, false, false);
        opacityLabel.setLayoutData(gridData);

        Combo opacityCombo = new Combo(compBottomLeftButtons, SWT.BORDER | SWT.READ_ONLY);
        pageControls.put(COMBO_OPACITY, opacityCombo);
        opacity = new double[11];
        for (int i = 0; i < 11; i++) {
            opacityCombo.add(Integer.toString(i*10)+"%"); //$NON-NLS-1$
            opacity[i] = i / 10.0;
        }
        opacityCombo.select(5); //default 50%
        opacityCombo.setVisibleItemCount(8);
        opacityCombo.addSelectionListener(new SimpleComboListener());

        Button reverseButton = new Button(compBottomLeftButtons, SWT.RIGHT);
        reverseButton.setText(Messages.StyleEditor_theme_reverse); 
        reverseButton.addSelectionListener(new SelectionListener() {

            public void widgetSelected( SelectionEvent e ) {
                reverseColours = !reverseColours; //used by generateTheme to get things half right
                FeatureTypeStyle fts = getFTS();
                Rule[] rule = fts.getRules();
                for (int i = 0; i < (rule.length / 2); i++) {
                    swapColours(rule[i], rule[rule.length-i-1]);
                }
                treeViewer.refresh();
            }

            public void widgetDefaultSelected( SelectionEvent e ) {
                widgetSelected(e);
            }
        
        });
        
        Button removeButton = new Button(compBottomLeftButtons, SWT.RIGHT);
        removeButton.setText(Messages.StyleEditor_theme_remove); 
        removeButton.setEnabled(false);
        removeButton.addSelectionListener(new SelectionListener() {

            public void widgetSelected( SelectionEvent e ) {
                StructuredSelection selection = (StructuredSelection) treeViewer.getSelection();
                if (selection.isEmpty()) return;
                Rule rule = (Rule) selection.getFirstElement();
                if (removeRule(rule)) {
                    treeViewer.refresh();
                } else {
                    //freak out
                }
            }

            public void widgetDefaultSelected( SelectionEvent e ) {
                widgetSelected(e);
            }
        
        });
        pageControls.put(BUTTON_REMOVE, removeButton);
        
        treeViewer = new TreeViewer(compBottomLeft, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
        Tree treeTable = treeViewer.getTree();
        treeTable.setHeaderVisible(true);
        treeTable.setLinesVisible(true);
        gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
        gridData.widthHint = compBottomLeft.getBounds().x;
        gridData.heightHint = compBottomLeft.getBounds().y;
        treeTable.setLayoutData(gridData);
        
        TreeColumn colImage = new TreeColumn(treeTable, SWT.LEFT);
        colImage.setText(Messages.StyleEditor_theme_column_colour); 

        TreeColumn colTitle = new TreeColumn(treeTable, SWT.LEFT);
        colTitle.setText(Messages.StyleEditor_theme_column_label); 
        
        TreeColumn colExpr = new TreeColumn(treeTable, SWT.LEFT);
        colExpr.setText(Messages.StyleEditor_theme_column_expression); 
        
        treeTable.layout();
        
        tableSettings = new TableSettings(treeTable);
        tableSettings.setColumnMin(0, 40);
        tableSettings.setColumnMin(1, 100);
        tableSettings.setColumnMin(0, 50);
        
        treeViewer.setLabelProvider(new StyleTreeLabelProvider());
        treeViewer.setSorter(new StyleTreeSorter());
        treeViewer.setContentProvider(new StyleTreeContentProvider());

        CellEditor[] editors = new CellEditor[3];

        //TODO: create a nicer color chooser
        CellEditor celledit0 = new ColorCellEditor(treeViewer.getTree());
        TextCellEditor celledit1 = new TextCellEditor(treeViewer.getTree()); 
        TextCellEditor celledit2 = new TextCellEditor(treeViewer.getTree()); 
        editors[0] = celledit0;
        editors[1] = celledit1;
        editors[2] = celledit2;
        
        treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {

            public void selectionChanged( SelectionChangedEvent event ) {
                
                if (((StructuredSelection) event.getSelection()).isEmpty()) {
                    getButton(BUTTON_REMOVE).setEnabled(false);
                } else {
                    getButton(BUTTON_REMOVE).setEnabled(true);
                }
                
            }
            
        });
        
        treeViewer.setCellModifier(new ICellModifier() {
            
            public boolean canModify(Object element, String property) {
                if (property.equals("styleExpr")) { //$NON-NLS-1$
                    if (element instanceof Rule) {
                        Rule rule = (Rule) element;
                        if (rule.getName().startsWith("rule")) { //$NON-NLS-1$
                            if (rule.hasElseFilter()) {
                                return false;
                            }
                            return true;
                        } else {
                            return false;
                        }
                    }
                } else if (property.equals("title")) { //$NON-NLS-1$
                    return true;
                } else if (property == "colour") { //$NON-NLS-1$
                    return true;
                }
                return false;
            }

            public Object getValue(Object element, String property) {
                if (property.equals("styleExpr")) { //$NON-NLS-1$
                    if (element instanceof Rule) {
                        Rule rule = (Rule) element;
                        if (rule.hasElseFilter()) {
                            return "else"; //$NON-NLS-1$
                        } else {
                            return StyleGenerator.toStyleExpression(rule.getFilter());
                        }
                    }
                } else if (property.equals("title")) { //$NON-NLS-1$
                    if (element instanceof Rule) {
                        Rule rule = (Rule) element;
                        //the title contains the same value, but we'll obtain it from the rule anyways
                        return rule.getTitle();
                    }
                } else if (property.equals("colour")) { //$NON-NLS-1$
                    if (element instanceof Rule) {
                        Rule rule = (Rule) element;
                        Color colour = SLDs.toColor(SLDs.colors(rule)[0]);
                        RGB rgb = new RGB(colour.getRed(), colour.getGreen(), colour.getBlue());
                        return rgb;
                    }
                }

                return new Object[0];
            }

            public void modify(Object element, String property, Object value) {
                if (property.equals("styleExpr")) { //$NON-NLS-1$
                    if (element instanceof TreeItem) {
                        String newExpr = (String) value;
                        TreeItem item = (TreeItem) element;
                        Object data = item.getData();
                        
                        //create/modify the custom break and regen
                        //TODO: implement custom break classifier
                        //customBreak = new CustomClassifierFunction();

                        //TODO: set expression?
                        //figure out which rule has changed
                        int ruleIndex = Integer.parseInt(((Rule) data).getName().substring(4))-1;
                        //fill the custom classifier with the rule boundary values
                        Rule[] rules = getFTS().getRules();
                        for (int i = 0; i < rules.length; i++) {
                            String thisExpr;
                            if (i == ruleIndex) { //use the new value
                                thisExpr = newExpr;
                            } else { //use the fts value
                                if (rules[i].getFilter() == null) {
                                    thisExpr = null; //probably an else filter
                                } else {
                                    thisExpr = StyleGenerator.toStyleExpression(rules[i].getFilter());
                                }
                            }
                            if (thisExpr == null) {
                                //TODO: mark as "else"
                            } else if (StyleGenerator.isRanged(thisExpr)) {
                                String[] minMax = thisExpr.split("\\.\\."); //$NON-NLS-1$
                                Double min = new Double(minMax[0]);
                                Double max = new Double(minMax[1]);
                                //customBreak.setRangedValues(i, min, max);
                            } else {
                                String[] values = thisExpr.split(", "); //$NON-NLS-1$
                                for (int j = 0; j < values.length; j++) {
                                    //customBreak.setExplicitValues(i, values[j]);
                                }
                            }
                        }
                        Combo breaks = getCombo(COMBO_BREAKTYPE);
                        String[] allBreaks = breaks.getItems();
                        int hasCustom = -1;
                        for (int i = 0; i < allBreaks.length; i++) {
                            if (allBreaks[i].equalsIgnoreCase(Messages.StyleEditor_theme_custom)) { 
                                hasCustom = i;
                                break;
                            }
                        }
                        if (hasCustom > -1) {
                            breaks.select(hasCustom);
                        } else {
                            breaks.add(Messages.StyleEditor_theme_custom); 
                            breaks.select(allBreaks.length);
                        }
                        if (inputsValid()) generateTheme();
                        //old method: update the fts
//                        int index = item.getParent().indexOf(item);
//                        //TODO: check that style conforms to our standard
//                        FeatureTypeStyle fts = getFTS();
//                        try {
//                            StyleGenerator.modifyFTS(fts, index, (String) value);
//                        } catch (IllegalFilterException e) {
//                            // TODO Handle IllegalFilterException
//                            throw new RuntimeException(e.getMessage());
//                        }
//                        treeViewer.setInput(fts);
                    }
                } else if (property.equals("title")) { //$NON-NLS-1$
                    if (element instanceof TreeItem) {
                        TreeItem item = (TreeItem) element;
                        Object data = item.getData();
                        if (data instanceof Rule) {
                            Rule rule = (Rule) data;
                            if (rule.getTitle().equals(value)) return; //don't bother -- value is the same
                            rule.setTitle(value.toString());
                        }
                        treeViewer.refresh();
                    }
                } else if (property.equals("colour")) { //$NON-NLS-1$
                    if (value == null) return;
                    if (element instanceof TreeItem) {
                        TreeItem item = (TreeItem) element;
                        Object data = item.getData();
                        Expression newColorExpr = null;
                        Expression oldColorExpr = null;
                        Color newColor = null;
                        StyleBuilder sb = new StyleBuilder();
                        if (value instanceof RGB) {
                            RGB rgb = (RGB) value;
                            newColor = new Color(rgb.red, rgb.green, rgb.blue);
                            String htmlColor = SLDs.toHTMLColor(newColor);
                            newColorExpr = sb.literalExpression(htmlColor);
                        }
                        if (data instanceof Rule) {
                            Rule rule = (Rule) data;
                            Symbolizer[] symb = rule.getSymbolizers();
                            if (symb.length == 1) { //we're only expecting 1
                                if (symb[0] instanceof PolygonSymbolizer) {
                                    PolygonSymbolizer ps = (PolygonSymbolizer) symb[0];
                                    Fill fill = ps.getFill();
                                    oldColorExpr = fill.getColor();
                                    if (oldColorExpr.equals(newColorExpr)) {
                                        return; //don't bother, same colour
                                    }
                                    //determine if the palette is already customized
                                    if (customPalette == null) {
                                        int numClasses = new Integer(getCombo(COMBO_CLASSES).getItem(getCombo(COMBO_CLASSES).getSelectionIndex())).intValue();
                                        //create the palette from the current one
                                        BrewerPalette pal = (BrewerPalette) ((StructuredSelection) paletteTable.getSelection()).getFirstElement(); 
                                        customPalette = new BrewerPalette();
                                        PaletteSuitability suitability = new PaletteSuitability();
                                        //suitability.
                                        //customPalette.setColors()
                                        SampleScheme newScheme = new SampleScheme();
                                        Color[] colors = new Color[pal.getMaxColors()];
                                        Color[] allColorsArray = pal.getColors();
                                        List<Color> allColors = new ArrayList<Color>();
                                        for (int i = 0; i < allColorsArray.length; i++) {
                                            allColors.add(allColorsArray[i]);
                                        }
                                        String unknown = "?"; //$NON-NLS-1$
                                        for (int i = 0; i < pal.getMaxColors(); i++) {
                                            if (i > 0) { 
                                                //create a simple scheme
                                                int[] scheme = new int[i+1];
                                                for (int j = 0; j < i+1; j++) {
                                                    scheme[j] = j;
                                                }
                                                newScheme.setSampleScheme(i+1, scheme);
                                                //set the suitability to unknown
                                                try {
                                                    suitability.setSuitability(i+1, new String[] {unknown, unknown, unknown, unknown, unknown, unknown});
                                                } catch (IOException e) {
                                                    SLDPlugin.log("setSuitability() failed", e); //$NON-NLS-1$
                                                    return;
                                                }
                                            }
                                            //copy the color
                                            if (i < numClasses) {
                                                //copy the colors directly over
                                                colors[i] = pal.getColor(i, numClasses);
                                                allColors.remove(colors[i]);
                                            } else {
                                                //find unique colors to fill in the rest of the palette
                                                colors[i] = allColors.remove(0);
                                            }
                                        }
                                        //newScheme.setSampleScheme(3, new int[] {0,1});
                                        customPalette.setPaletteSuitability(suitability);
                                        customPalette.setColors(colors);
                                        customPalette.setColorScheme(newScheme);
                                        customPalette.setName(Messages.StyleEditor_theme_custom); 
                                        customPalette.setDescription(Messages.StyleEditor_theme_custom_desc); 
                                        customPalette.setType(pal.getType());
                                        if (!getBrewer().hasPalette(Messages.StyleEditor_theme_custom)) {
                                            getBrewer().registerPalette(customPalette);
                                        }
                                    }
                                    //seek and destroy the old colour
                                    Color[] colors = customPalette.getColors();
                                    int expectedIndex = -1;
                                    if (rule.getName().toLowerCase().startsWith("rule")) { //$NON-NLS-1$
                                        expectedIndex = Integer.parseInt(rule.getName().substring(4))-1;
                                    }
                                    int actualIndex = -1;
                                    for (int i = 0; i < colors.length; i++) {
                                        if (oldColorExpr.equals(sb.literalExpression(SLDs.toHTMLColor(colors[i])))) {
                                            actualIndex = i;
                                            if (expectedIndex == i) {
                                                //we found the correct old color where we expected to
                                                break;
                                            } //otherwise, keep looking just in case
                                        }
                                    }
                                    if (actualIndex == -1) {
                                        SLDPlugin.log("color match unsuccessful... "+oldColorExpr+" vs "+newColorExpr, null); //$NON-NLS-1$ //$NON-NLS-2$
                                        //TODO: use different colour matching technique
                                        return;
                                    }
                                    colors[actualIndex] = newColor;
                                    customPalette.setColors(colors);
                                    paletteTable.setInput(getBrewer());
                                    paletteTable.setSelection(new StructuredSelection(customPalette));
                                }
                            }
                        }
                        treeViewer.refresh();
                    }
                }

            }
        });
        treeViewer.setColumnProperties(new String[] {"colour", "title", "styleExpr"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        treeViewer.setCellEditors(editors);
        //populate the tree
        treeViewer.setInput(getFTS());
        treeViewer.expandAll();
        Control treeControl = treeViewer.getControl();
        GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
        gd.horizontalSpan = 2;
        treeControl.setLayoutData(gd);
        treeControl.setFont(font);
        treeControl.setVisible(false);
        
        Composite compBottomRight = new Composite(compBottom, SWT.NONE);
        compBottomRight.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false));
        layout = new GridLayout(1, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        compBottomRight.setLayout(layout);
        
        Label labelSuitability = new Label(compBottomRight, SWT.NONE);
        labelSuitability.setLayoutData(new GridData(SWT.NONE, SWT.NONE, false, false));
        labelSuitability.setText(Messages.StyleEditor_theme_suitability); 
        
        Composite icons = new Composite(compBottomRight, SWT.RIGHT);
        icons.setLayoutData(new GridData(SWT.RIGHT, SWT.NONE, false, false));
        layout = new GridLayout(3, true);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        icons.setLayout(layout);

        createSuitabilityIcon(icons, LABEL_ICON_COLORBLIND, Messages.StyleEditor_theme_suitability_colour, Images.getDescriptor(ImageConstants.COLORBLIND_ICON).createImage()); 
        createSuitabilityIcon(icons, LABEL_ICON_CRT, Messages.StyleEditor_theme_suitability_crt, Images.getDescriptor(ImageConstants.CRT_ICON).createImage()); 
        createSuitabilityIcon(icons, LABEL_ICON_PROJECTOR, Messages.StyleEditor_theme_suitability_proj, Images.getDescriptor(ImageConstants.PROJECTOR_ICON).createImage()); 
        createSuitabilityIcon(icons, LABEL_ICON_LCD, Messages.StyleEditor_theme_suitability_lcd, Images.getDescriptor(ImageConstants.LAPTOP_ICON).createImage()); 
        createSuitabilityIcon(icons, LABEL_ICON_PRINT, Messages.StyleEditor_theme_suitability_print, Images.getDescriptor(ImageConstants.PRINTER_ICON).createImage()); 
        createSuitabilityIcon(icons, LABEL_ICON_PHOTOCOPY, Messages.StyleEditor_theme_suitability_pcopy, Images.getDescriptor(ImageConstants.PHOTOCOPY_ICON).createImage()); 
        
        //create status bar
        Label status = new Label(parent, SWT.NONE);
        gridData = new GridData(SWT.FILL, SWT.BOTTOM, true, false);
        status.setLayoutData(gridData);
        status.setVisible(true);
        status.setFont(font);
        status.setText(""); //$NON-NLS-1$
        pageControls.put(LABEL_STATUSBAR, status);
    }

    private boolean removeRule(Rule rule) {
        //find the FTS containing the rule
        GTComponent parent = rule.getNote().getParent();
        if (!(parent instanceof FeatureTypeStyle)) {
            return false;
        }
        FeatureTypeStyle fts = (FeatureTypeStyle) parent;
        Rule[] rules = fts.getRules();
        boolean found = false;
        int index = 0;
        for (index = 0; index < rules.length; index++) {
            if (rules[index] == rule) {
                found = true;
                break;
            }
        }
        if (!found) return false;
        Object[] result = removeElement(rules, index);
        Rule[] newRules = new Rule[result.length];
        for (int i = 0; i < newRules.length; i++) {
            newRules[i] = (Rule) result[i];
        }
        fts.setRules(newRules);
        return true;
    }
    
    private void swapColours(Rule rule1, Rule rule2) {
        Symbolizer[] symb1 = rule1.getSymbolizers();
        Symbolizer[] symb2 = rule2.getSymbolizers();
        if (symb1.length != symb2.length) {
            SLDPlugin.log("StyleThemePage.swapColours(): Number of symbolizers each rule and not equal - aborting colour swap", null); //$NON-NLS-1$
            return;
        }
        Expression tempColour; 
        for (int i = 0; i < symb1.length; i++) {
            if (symb1[i] instanceof PolygonSymbolizer) {
                tempColour = ((PolygonSymbolizer) symb1[i]).getFill().getColor();
                ((PolygonSymbolizer) symb1[i]).getFill().setColor(((PolygonSymbolizer) symb2[i]).getFill().getColor());
                ((PolygonSymbolizer) symb2[i]).getFill().setColor( tempColour);
            } else if (symb1[i] instanceof PointSymbolizer) {
                tempColour = ((PointSymbolizer) symb1[i]).getGraphic().getMarks()[0].getFill().getColor();
                ((PointSymbolizer) symb1[i]).getGraphic().getMarks()[0].getFill().setColor(((PointSymbolizer) symb2[i]).getGraphic().getMarks()[0].getFill().getColor());
                ((PointSymbolizer) symb2[i]).getGraphic().getMarks()[0].getFill().setColor( tempColour);
            } else if (symb1[i] instanceof LineSymbolizer) {
                tempColour = ((LineSymbolizer) symb1[i]).getStroke().getColor();
                ((LineSymbolizer) symb1[i]).getStroke().setColor(((LineSymbolizer) symb2[i]).getStroke().getColor());
                ((LineSymbolizer) symb2[i]).getStroke().setColor( tempColour);
            }
        }
    }
    
    public void setStatusText(String text) {
        getLabel(LABEL_STATUSBAR).setText(text);
    }
    
    @Override
    public String getLabel() {
        return null;
    }
    
    private class AttributeComboListener implements SelectionListener {
        public void widgetSelected(SelectionEvent e) {
            //record the attribute
            selectedAttributeType = getAttributeType(getCombo(COMBO_ATTRIBUTES).getText());
            //update the normalization combo
            updateNormalize();
            //update the classification type (breaks) combo
            updateBreaks();
            if (isNumber(selectedAttributeType)) {
                //clear the status label
                setStatusText(""); //$NON-NLS-1$
            } else {
                //TODO: don't use strings to match break types
                if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_uniques)) { 
                    //determine if we've already calculated the number of unique values
                    String attribute = selectedAttributeType.getName();
                    if (uniqueCounts.containsKey(attribute)) {
                        updateUnique(attribute, uniqueCounts.get(attribute));
                    }
                    //count the number of unique attributes
                    int uniqueCount = -1;
                    try {
                        UniqueVisitor uniques = new UniqueVisitor(attribute);
                        ProgressListener progress = new NullProgressListener();
                        collection.accepts(uniques, (org.geotools.util.ProgressListener) progress);
                        uniqueCount = uniques.getUnique().size();
                    } catch (IOException e1) {
                        SLDPlugin.log("unique values calculation failed", e1); //$NON-NLS-1$
                    }
                    if (uniqueCount > -1) {
                        updateUnique(attribute, uniqueCount);                    
                    }
                } else {
                    //clear the status label
                    setStatusText(""); //$NON-NLS-1$
                }
            }

            //calculate the color scheme, if we have enough information
            if (inputsValid()) generateTheme();
            return;
        }

        public void widgetDefaultSelected(SelectionEvent e) {
            widgetSelected(e);
        }

        private void updateUnique(String attr, int uniqueCount) {
            setStatusText(MessageFormat.format(Messages.StyleEditor_theme_unique_values, uniqueCount, attr));
            if (uniqueCount < 12) {
                if (getCombo(COMBO_ELSE).getSelectionIndex() > 0) {
                    getCombo(COMBO_CLASSES).select(uniqueCount - 1);
                } else {
                    getCombo(COMBO_CLASSES).select(uniqueCount - 2);
                }
            } else if (uniqueCount == 12) {
                getCombo(COMBO_CLASSES).select(10);
            }
        }
    }

    private class PalettesListener implements ISelectionChangedListener {

        public void selectionChanged(SelectionChangedEvent event) {
            if (inputsValid()) generateTheme();
        }
        
    }

    private class SuitabilityToggleListener implements SelectionListener {
        private int id;
        
        public SuitabilityToggleListener(int id) {
            super();
            this.id = id;
        }

        public void widgetSelected( SelectionEvent e ) {
            System.out.println(e.getSource());
            updateToggleTooltip(id);
            updatePalettes();
        }

        public void widgetDefaultSelected( SelectionEvent e ) {
            widgetSelected(e);
        }
        
    }
    
    private class ClassesComboListener implements SelectionListener {
        public void widgetSelected(SelectionEvent e) {
            updatePalettes();
            if (inputsValid()) generateTheme();
            return;
        }

        public void widgetDefaultSelected(SelectionEvent e) {
            widgetSelected(e);
        }

    }
    
    private class ElseComboListener implements SelectionListener {
        public void widgetSelected(SelectionEvent e) {
            int index = getCombo(COMBO_ELSE).getSelectionIndex();
            int classesIndex = getCombo(COMBO_CLASSES).getSelectionIndex();
            if (elseSelection && index == 0) { //else rule was turned off
                if (classesIndex > 0) {
                    getCombo(COMBO_CLASSES).select(classesIndex - 1);
                }
            } else if (!elseSelection && index != 0) { //else rule was turned on
                if (classesIndex < (getCombo(COMBO_CLASSES).getItemCount() - 1)) {
                    getCombo(COMBO_CLASSES).select(classesIndex + 1);
                }
            } 
            elseSelection = (index > 0);
            if (inputsValid()) generateTheme();
        }

        public void widgetDefaultSelected(SelectionEvent e) {
            widgetSelected(e);
        }
    }
    
    private class SimpleComboListener implements SelectionListener {
        public void widgetSelected(SelectionEvent e) {
            if (inputsValid()) generateTheme();
        }

        public void widgetDefaultSelected(SelectionEvent e) {
            widgetSelected(e);
        }
    }

    private void updatePalettes() {
        paletteTable.setInput(getBrewer());
    }

    private void updateBreaks() {
        Combo breaksCombo = getCombo(COMBO_BREAKTYPE);
        String currentAttr = getCombo(COMBO_ATTRIBUTES).getText();
        String value = breaksCombo.getText();
        breaksCombo.removeAll();
        if (isNumber(currentAttr)) { //show numeric break types
            breaksCombo.add(Messages.StyleEditor_theme_equalInterval); 
            breaksCombo.add(Messages.StyleEditor_theme_quantile); 
            //breaksCombo.add(Messages.StyleEditor_theme_standardDeviation); 
            int index = breaksCombo.indexOf(value);
            if (index > -1) breaksCombo.select(index);
            else breaksCombo.select(1);
        } else { //show categorical break types
            breaksCombo.add(Messages.StyleEditor_theme_uniques); 
            int index = breaksCombo.indexOf(value);
            if (index > -1) breaksCombo.select(index);
            else breaksCombo.select(0);
        }
    }
    
    private void updateNormalize() {
        Combo normalizeCombo = getCombo(COMBO_NORMALIZE);
        String value = normalizeCombo.getText();
        String currentAttr = getCombo(COMBO_ATTRIBUTES).getText();
        normalizeCombo.removeAll();
        normalizeCombo.add(Messages.StyleEditor_theme_none); 
        if (isNumber(currentAttr)) {
            normalizeCombo.setEnabled(true);
            for (int i = 0; i < numericAttr.size(); i++) {
                String attr = numericAttr.get(i);
                if (!currentAttr.equals(attr))
                    normalizeCombo.add(attr);
            }
            int index = normalizeCombo.indexOf(value);
            if (index > 1) normalizeCombo.select(index);
            else normalizeCombo.select(0); //default is "< NONE >"
        } else {
            normalizeCombo.select(0);
            normalizeCombo.setEnabled(false);
        }
    }
    
    private boolean inputsValid() {
        if (getCombo(COMBO_ATTRIBUTES).getText().length() < 1) return false;
        else if (paletteTable.getSelection().isEmpty()) return false;
        else if (getCombo(COMBO_CLASSES).getText().length() < 1) return false;
        else return true;
    }

    /**
     * This method is called every time you do something; its job is to make the Style based on your settings.
     * If the parameters have changed we will make a new classifier.
     */
    private void generateTheme() {
        if (selectedAttributeType == null) {
            selectedAttributeType = getAttributeType(getCombo(COMBO_ATTRIBUTES).getText());
        }
        if (getCombo(COMBO_NORMALIZE).getSelectionIndex() == 0) {
            normalize = null;
        } else {
            normalize = getAttributeType(getCombo(COMBO_NORMALIZE).getText());
        }
        
        //change the close button to cancel
        ((StyleEditorDialog) getContainer()).setCancelMode(true);
        
        //generate the defaults
        Runnable genDefault = new Runnable(){
            public void run() {
                //generate the expression
                org.geotools.filter.Expression expr;
                try {
                    Expression attr = ff.property(selectedAttributeType.getName());
                    if (normalize != null) {
                        Expression divide = ff.divide(attr, ff.property(normalize.getName()));
                        expr = (org.geotools.filter.Expression) divide;
                    } else {
                        expr = (org.geotools.filter.Expression) attr;
                    }
                } catch (IllegalFilterException e) {
                    // TODO Handle IllegalFilterException
                    throw (RuntimeException) new RuntimeException( ).initCause( e );
                }
                BrewerPalette pal = (BrewerPalette) ((StructuredSelection) paletteTable.getSelection()).getFirstElement();
                String paletteName = pal.getName();
                int numClasses = new Integer(getCombo(COMBO_CLASSES).getItem(getCombo(COMBO_CLASSES).getSelectionIndex())).intValue();
                int[] suitability = pal.getPaletteSuitability().getSuitability(numClasses);
                
                //check for custom classifier
                if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_custom)) { 
                    //classifier = customBreak; // If the user has started hacked
                }                
                boolean classifierModified = false;
                if (classifierFunc == null || classifier == null) {
                    classifierModified = true;
                } else {
                    //determine if the classifier needs recalculation
                    if (classifierFunc.getNumberOfClasses() != new Integer(getCombo(COMBO_CLASSES).getText()).intValue()) {
                        if (getCombo(COMBO_ELSE).getSelectionIndex() == 0) classifierModified = true;
                        if (classifierFunc.getNumberOfClasses() != new Integer(getCombo(COMBO_CLASSES).getText()).intValue()-1)
                            classifierModified = true;
                    }
                    if (getCombo(COMBO_ELSE).getSelectionIndex() > 0) 
                        classifierModified = true;
                    else if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_custom))  
                        classifierModified = true;
                    else if ((getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_equalInterval)) && !(classifierFunc instanceof EqualIntervalFunction)) 
                        classifierModified = true;
                    else if ((getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_quantile)) && !(classifierFunc instanceof QuantileFunction)) 
                        classifierModified = true;
                    else if ((getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_standardDeviation)) && !(classifierFunc instanceof StandardDeviationFunction)) 
                        classifierModified = true;
                    else if (!classifierFunc.getExpression().equals(expr))
                        classifierModified = true;
                }
                
                String semanticTypeIdentifier;
                //break type:palette
                if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_uniques)) 
                    semanticTypeIdentifier = "unique:"; //$NON-NLS-1$
                else if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_equalInterval)) 
                    semanticTypeIdentifier = "equalinterval:"; //$NON-NLS-1$
                else if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_quantile)) 
                    semanticTypeIdentifier = "quantile:"; //$NON-NLS-1$
                else if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase("standard deviation")) //$NON-NLS-1$
                    semanticTypeIdentifier = "standarddeviation:"; //$NON-NLS-1$
                else if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_custom)) 
                    semanticTypeIdentifier = "custom:"; //$NON-NLS-1$
                else {
                    semanticTypeIdentifier = "default:"; //$NON-NLS-1$
                }
                semanticTypeIdentifier = semanticTypeIdentifier.concat(paletteName.toLowerCase());
                
                //create the classification function, if necessary
                if (classifierModified) {
                    //TODO: add other classifiers
                    if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_uniques)) 
                        classifierFunc = new UniqueIntervalFunction();
                    else if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_equalInterval)) 
                        classifierFunc = new EqualIntervalFunction();
                    else if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_quantile)) 
                        classifierFunc = new QuantileFunction();
                    else if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_standardDeviation)) 
                        classifierFunc = new StandardDeviationFunction();
                    //else if (getCombo(COMBO_BREAKTYPE).getText().equalsIgnoreCase(Messages.StyleEditor_theme_custom)) 
                        //classifier = customBreak;
                    else {
                        return; // if you manage to choose something we do not understand.
                    }
                    ProgressListener cancelProgress = ((StyleEditorDialog) getContainer()).getProgressListener();
                    classifierFunc.setProgressListener((org.geotools.util.ProgressListener) cancelProgress);
                    numClasses = new Integer(getCombo(COMBO_CLASSES).getText()).intValue();
                    if (getCombo(COMBO_ELSE).getSelectionIndex() > 0) {
                    	numClasses--;
                    }
                    //pass parameters
                    List params = new ArrayList();
                    params.add(0, expr); //expression
                    params.add(1, ff.literal(numClasses)); //classes
                    classifierFunc.setParameters(params);
                    
                    classifier = (Classifier) classifierFunc.evaluate(collection);
                }

                //generate the style
                Color[] colors = getBrewer().getPalette(paletteName).getColors(numClasses);
                if (reverseColours) {
                    for (int i = 0; i < colors.length / 2; i++) {
                        Color tempColor = colors[i];
                        int j = colors.length-i-1;
                        colors[i] = colors[j];
                        colors[j] = tempColor;
                    }
                }
                
              int opacIndex = getCombo(COMBO_OPACITY).getSelectionIndex();
              double opac;
              if (opacIndex > -1) {
                  opac = opacity[opacIndex];
              } else {
                  opac = 1;
              }
              //TODO: justify outline colour (read existing stroke)
              Stroke stroke = sb.createStroke(Color.BLACK, 1, opac);
              int elseMode = StyleGenerator.ELSEMODE_IGNORE;
              if (getCombo(COMBO_ELSE).getSelectionIndex() == 0) {
            	  elseMode = StyleGenerator.ELSEMODE_IGNORE;
              } else if (getCombo(COMBO_ELSE).getSelectionIndex() == 1) {
            	  elseMode = StyleGenerator.ELSEMODE_INCLUDEASMIN;
              } else if (getCombo(COMBO_ELSE).getSelectionIndex() == 2) {
            	  elseMode = StyleGenerator.ELSEMODE_INCLUDEASMAX;
              }	
              GeometryAttributeType gat = getSelectedLayer().getSchema().getDefaultGeometry();
              
              FeatureTypeStyle newFTS = null;
              try {
            	  if( classifier == null ){
            		  System.out.println("WARNING");
            	  }
                    newFTS = StyleGenerator.createFeatureTypeStyle(classifier, expr, colors, semanticTypeIdentifier, gat, elseMode, opac, stroke);
				} catch (IllegalFilterException e) {
					newFTS = null;
					SLDPlugin.log("sg.createFeatureTypeStyle() failed", e); //$NON-NLS-1$
				} catch (NullPointerException e) {
					newFTS = null;
					SLDPlugin.log("sg.createFeatureTypeStyle() failed", e); //$NON-NLS-1$
				}
                
                if (newFTS == null) {
                    ErrorManager.get().displayError(Messages.StyleEditor_error, Messages.StyleEditor_theme_failure);
//                    MessageDialog
//                    .openError(
//                            PlatformUI.getWorkbench()
//                                    .getActiveWorkbenchWindow()
//                                    .getShell(),
//                                    Messages.StyleEditor_error, 
//                                    Messages.StyleEditor_theme_failure); 
                    return;
                } else {
                    //set the FeatureTypeName to the current layer name
                    newFTS.setFeatureTypeName(getSelectedLayer().getSchema().getTypeName());
                    //get the style
                    Style style = getStyle();
                    //ensure the style has an SLD
                    if (style == null) throw new RuntimeException("Style is null"); //$NON-NLS-1$
                    StyledLayerDescriptor sld = SLDs.styledLayerDescriptor(style);
                    if (sld == null) {
                        SLDContent.createDefaultStyledLayerDescriptor(style);
                        sld = SLDs.styledLayerDescriptor(style);
                        if (sld == null) throw new RuntimeException("SLD is null"); //$NON-NLS-1$
                    }
                    //insert/replace the FTS
                    try {
                        addThemedFTStoStyle(style, newFTS);
                    } catch (Exception e) {
                        String msg = "addThemedFTStoStyle() failed"; //$NON-NLS-1$
                        SLDPlugin.log(msg, e);
                        ErrorManager.get().displayException(e, msg, SLDPlugin.ID);
                        return;
                    }
                    
                    //update the suitability icons
                    updateSuitabilities(suitability);
                    
                    treeViewer.setInput(newFTS);
                }
            }
        };
        try {
            BusyIndicator.showWhile(Display.getCurrent(), genDefault);
        } catch (Exception e) {
            String msg = "Theme Generation Failed"; //$NON-NLS-1$
            SLDPlugin.log(msg, e);
            ErrorManager.get().displayException(e, msg, SLDPlugin.ID);
        }
        
        //turn cancel off; go back to "close"
        ((StyleEditorDialog) getContainer()).setCancelMode(false);

        //resize the palette selection table
        displayBottomComposite();
        TableUtils.resizeColumns(treeViewer.getTree(), tableSettings, TableUtils.MODE_JUMP);
        //TODO: only the first time
        
        treeViewer.getControl().setVisible(true);
    }

    /**
     * Iterates through the style blackboard style and returns the ColorBrewer FeatureTypeStyle.
     *
     * @return
     */
    private FeatureTypeStyle getFTS() {
        Style style = getStyle();
        FeatureTypeStyle[] fts = style.getFeatureTypeStyles();
        for (int i = 0; i < fts.length; i++) {
            if (SLDs.isSemanticTypeMatch(fts[i], "colorbrewer:.*")) { //$NON-NLS-1$
                return fts[i];
            }
        }
        return null;
    }
    
    /**
     * Given a Style object and a FeatureTypeStyle, this method scans the SemanticTypeIdentifiers of
     * each FTS inside the style. If a ColorBrewer themed identifier is found, the new FTS will
     * replace it.
     * 
     * @param style future parent of the FTS
     * @param fts FTS to insert/replace with
     */
    private void addThemedFTStoStyle(Style style, FeatureTypeStyle fts) {
        //determine what symbolizers exist in the fts
        Set<Object> symbs = new HashSet<Object>();
        Rule[] newRules = fts.getRules();
        for (int i = 0; i < newRules.length; i++) {
            Symbolizer[] newSymbs = newRules[i].getSymbolizers();
            for (int j = 0; j < newSymbs.length; j++) {
                symbs.add(newSymbs[j].getClass());
            }
        }
        
        FeatureTypeStyle[] FTSs = style.getFeatureTypeStyles();
        //search for a match, and replace if found
        boolean found = false;
        for (int i = 0; i < FTSs.length; i++) {
            if (SLDs.isSemanticTypeMatch(FTSs[i], "colorbrewer:.*")) { //$NON-NLS-1$
                FTSs[i] = fts;
                found = true;
            } else {
                //purge any conflicting Symbolizers (same class)
                //TODO: add conditions under which other Symbolizers might live
                Rule[] rule = FTSs[i].getRules();
                for (int j = 0; j < rule.length; j++) {
                    Symbolizer[] symb = rule[j].getSymbolizers();
                    Symbolizer[] newSymb = symb.clone();
                    int deletedElements = 0;
                    for (int k = 0; k < symb.length; k++) {
                        if (symbs.contains(symb[k].getClass())) { //the Symbolizer class is a match
                            Object[] temp = removeElement(newSymb, k-deletedElements);
                            if (temp.length > 0) { 
                                newSymb = new Symbolizer[temp.length];
                                for (int l = 0; l < temp.length; l++) {
                                    newSymb[l] = (Symbolizer) temp[l];
                                }
                            } else {
                                newSymb = new Symbolizer[0];
                            }
                            deletedElements++;
                        }
                    }
                    if (deletedElements > 0) {
                        rule[j].setSymbolizers(newSymb);
                    }
                }
            }
        }
        //delete any rules with zero symbolizers
        for (int i = 0; i < FTSs.length; i++) {
            boolean changed = false;
            Rule[] rule = FTSs[i].getRules();
            for (int j = 0; j < rule.length; j++) {
                if (rule[j].getSymbolizers().length == 0) {
                    if (rule.length > 1) {
                        Object[] temp = removeElement(rule, j);
                        rule = new Rule[temp.length];
                        for (int k = 0; k < temp.length; k++) {
                            rule[k] = (Rule) temp[k];
                        }
                    } else {
                        rule = new Rule[0];
                    }
                    changed = true;
                    j--;
                }
            }
            if (changed) {
                FTSs[i].setRules(rule);
            }
        }
        //delete any FTSs with zero rules
        for (int i = 0; i < FTSs.length; i++) {
            if (FTSs[i].getRules().length == 0) {
                Object[] temp = removeElement(FTSs, i);
                FTSs = new FeatureTypeStyle[temp.length];
                for (int j = 0; j < temp.length; j++) {
                    FTSs[j] = (FeatureTypeStyle) temp[j];
                }
            }
        }
        //create a FTS if needed
        if (!found) {
            //match was not found, so add the FTS
            FeatureTypeStyle[] newFTSList = new FeatureTypeStyle[FTSs.length+1];
            newFTSList[0] = fts; //add the new fts to the START of the array
            System.arraycopy(FTSs, 0, newFTSList, 1, FTSs.length);
            style.setFeatureTypeStyles(newFTSList);
        } else {
            style.setFeatureTypeStyles(FTSs);
        }
    }
    
    private Object[] removeElement(Object[] array, int indexToRemove) {
        if (array.length == 1) return new Object[0];
        Object[] newArray = new Object[array.length-1];
        if (indexToRemove > 0) {
            System.arraycopy(array, 0, newArray, 0, indexToRemove);
        }
        if (indexToRemove < array.length-1) {
            System.arraycopy(array, indexToRemove+1, newArray, indexToRemove, array.length-indexToRemove-1);
        }
        return newArray;
    }
        
    private AttributeType getAttributeType(String attributeTypeName) {
        FeatureType featureType = getSelectedLayer().getSchema();
        for (int i = 0; i < featureType.getAttributeCount(); i++) {
            AttributeType attributeType = featureType.getAttributeType(i);
            if (attributeType.getName().equals(attributeTypeName)) return attributeType;
        }
        return null;
    }

    private void createBrewer() {
        brewer = PlatformGIS.getColorBrewer();
    }

    @Override
    public void styleChanged( GTEvent arg ) {
    }
    
    public void gotFocus() {
    };
    
    private void displayBottomComposite() {
        Object layout = paletteTable.getControl().getLayoutData();
        if (layout instanceof GridData) {
            GridData gd = (GridData) layout;
            gd.grabExcessVerticalSpace = false;
            gd.heightHint = 70; //50 ~= 4 lines, 70 ~= 5 lines
        }
        getLabel(LABEL_SEPARATOR_BOTTOM).setVisible(true);
        getLayoutData(COMPOSITE_MIDDLE).grabExcessVerticalSpace = false;
        getComposite(COMPOSITE_BOTTOM).setVisible(true);
        getLayoutData(COMPOSITE_BOTTOM).grabExcessVerticalSpace = true;
        getLayoutData(COMPOSITE_BOTTOM_LEFT).grabExcessVerticalSpace = true;
        
        getComposite(COMPOSITE_MIDDLE).layout();
        getComposite(COMPOSITE_BOTTOM).layout();
        getComposite(COMPOSITE_PARENT).layout();
    }
    
    private void updateSuitabilities(int[] suitability) {
        for (Iterator i = viewerQuality.keySet().iterator(); i.hasNext();) {
            Object key = i.next();
            int icon = ((Integer) viewerQuality.get(key)).intValue();
            int quality = suitability[((Integer) key).intValue()];
            String viewer = Messages.StyleEditor_theme_suitability_visiblefor + " " //$NON-NLS-1$
                    + controlNames.get(icon) + " ";     //$NON-NLS-1$
            Label label = getLabel(icon);
            if (quality == PaletteSuitability.QUALITY_GOOD) {
                //getLabel(icon).setImage(getImageWithOverlay(icon, Images.getDescriptor(ImageConstants.GOOD_OVERLAY)));
                label.setImage(getImageWithOverlay(icon, null));
                label.setToolTipText(viewer+Messages.StyleEditor_theme_suitability_good); 
            } else if (quality == PaletteSuitability.QUALITY_DOUBTFUL) {
                getLabel(icon).setImage(getImageWithOverlay(icon, Images.getDescriptor(ImageConstants.DOUBTFUL_OVERLAY)));
                label.setToolTipText(viewer+Messages.StyleEditor_theme_suitability_doubtful); 
            } else if (quality == PaletteSuitability.QUALITY_BAD) {
                getLabel(icon).setImage(getImageWithOverlay(icon, Images.getDescriptor(ImageConstants.BAD_OVERLAY)));
                label.setToolTipText(viewer+Messages.StyleEditor_theme_suitability_bad); 
            } else { //UNKNOWN
                getLabel(icon).setImage(getImageWithOverlay(icon, Images.getDescriptor(ImageConstants.UNKNOWN_OVERLAY)));
                label.setToolTipText(viewer+Messages.StyleEditor_theme_suitability_unknown); 
            }
        }
    }
    
    private Image getImageWithOverlay(int iconLabelID, ImageDescriptor overlay) {
        //get the base image
        ImageDescriptor descriptor = null;
    	Image base = null;
        if (iconLabelID == LABEL_ICON_COLORBLIND) descriptor = Images.getDescriptor(ImageConstants.COLORBLIND_ICON); 
        else if (iconLabelID == LABEL_ICON_CRT) descriptor = Images.getDescriptor(ImageConstants.CRT_ICON); 
        else if (iconLabelID == LABEL_ICON_LCD) descriptor = Images.getDescriptor(ImageConstants.LAPTOP_ICON); 
        else if (iconLabelID == LABEL_ICON_PHOTOCOPY) descriptor = Images.getDescriptor(ImageConstants.PHOTOCOPY_ICON); 
        else if (iconLabelID == LABEL_ICON_PRINT) descriptor = Images.getDescriptor(ImageConstants.PRINTER_ICON); 
        else if (iconLabelID == LABEL_ICON_PROJECTOR) descriptor = Images.getDescriptor(ImageConstants.PROJECTOR_ICON); 
        if (descriptor != null) base = descriptor.createImage();
        if (overlay == null) return base;
        //apply the overlay
        DecoratorOverlayIcon ovrIcon = new DecoratorOverlayIcon(base, new ImageDescriptor[] {null,null,overlay,null,null});
        return ovrIcon.createImage();
    }

    public boolean performApply() {
        return true;
    }

    public void refresh() {
        //TODO: add refresh method
    }
}
