/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog;


import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import net.refractions.udig.catalog.internal.postgis.PostGISServiceImpl;
import net.refractions.udig.catalog.internal.postgis.PostgisPlugin;
import net.refractions.udig.catalog.postgis.internal.Messages;
import net.refractions.udig.core.internal.CorePlugin;

import org.geotools.data.DataStoreFactorySpi;
import org.geotools.data.postgis.PostgisDataStoreFactory;

/**
 * PostGis ServiceExtension
 * @author David Zwiers, Refractions Research
 * @since 0.6
 */
public class PostGISServiceExtension extends AbstractDataStoreServiceExtension implements ServiceExtension2 {

    /**
     * @param id
     * @param params
     * @return x
     */
    public IService createService( URL id, Map<String, Serializable> params ) {
        if (params != null && params.containsKey(getFactory().getParametersInfo()[2].key)
                && params.get(getFactory().getParametersInfo()[2].key) instanceof String) {
            String val = (String) params.get(getFactory().getParametersInfo()[2].key);
            params.remove(val);
            params.put(getFactory().getParametersInfo()[2].key, new Integer(val));
        }
        if (!getFactory().canProcess(params))
            return null;
        if (id == null) {
            try {
                URL toURL = toURL(params);
                return new PostGISServiceImpl(toURL, params);
            } catch (MalformedURLException e) {
                PostgisPlugin.log("Unable to construct proper service URL.", e); //$NON-NLS-1$
                return null;
            }
        }
        return new PostGISServiceImpl(id, params);
    }

    public static URL toURL( Map<String, Serializable> params ) throws MalformedURLException {
        String the_host = (String) params.get(PostGISServiceExtension.getFactory().getParametersInfo()[1].key);
        Integer intPort = (Integer) params.get(PostGISServiceExtension.getFactory().getParametersInfo()[2].key);
        String the_schema= (String) params.get(PostGISServiceExtension.getFactory().getParametersInfo()[3].key);
        String the_database = (String) params.get(PostGISServiceExtension.getFactory().getParametersInfo()[4].key);
        String the_username = (String) params.get(PostGISServiceExtension.getFactory().getParametersInfo()[3].key);
        String the_password = (String) params.get(PostGISServiceExtension.getFactory().getParametersInfo()[4].key);
        
        URL toURL = toURL(the_username, the_password, the_host, intPort, the_database, the_schema);
        return toURL;
    }

    /**
     * This is a guess ...
     * 
     * @see net.refractions.udig.catalog.ServiceExtension#createParams(java.net.URL)
     * @param url
     * @return x
     */
    public Map<String, Serializable> createParams( URL url ) {
        if (!isPostGIS(url)) {
            return null;
        }

        ParamInfo info=parseParamInfo(url);
        
        Map<String, Serializable> params = new HashMap<String, Serializable>();
        params.put(getFactory().getParametersInfo()[0].key, "postgis"); // dbtype //$NON-NLS-1$
        params.put(getFactory().getParametersInfo()[5].key, info.username); // user
        params.put(getFactory().getParametersInfo()[6].key, info.password); // pass 
        params.put(getFactory().getParametersInfo()[1].key, info.host); // host
        params.put(getFactory().getParametersInfo()[4].key, info.the_database); // database
        params.put(getFactory().getParametersInfo()[2].key, info.the_port); // port
        params.put(getFactory().getParametersInfo()[3].key, info.the_schema); // database
       return params;
    }

    private static PostgisDataStoreFactory factory = null;
    /**
     * @return x
     */
    public static PostgisDataStoreFactory getFactory() {
        if (factory == null) {
            // factory = new PostgisDataStoreFactory();
            factory = new PostgisDataStoreFactory();
        }
        return factory;
    }

    /** A couple quick checks on the url */
    public static final boolean isPostGIS( URL url ) {
        if( url==null )
            return false;
        return url.getProtocol().toLowerCase().equals("postgis") || url.getProtocol().toLowerCase().equals("postgis.jdbc") || //$NON-NLS-1$ //$NON-NLS-2$
        url.getProtocol().toLowerCase().equals("jdbc.postgis"); //$NON-NLS-1$
    }

    public static URL toURL( String the_username, String the_password, String the_host, Integer intPort, String the_database, String the_schema ) throws MalformedURLException {
        String the_spec = "postgis.jdbc://" + the_username //$NON-NLS-1$
                + ":" + the_password + "@" + the_host //$NON-NLS-1$ //$NON-NLS-2$
                + ":" + intPort + "/" + the_database //$NON-NLS-1$ //$NON-NLS-2$
                + "/" + the_schema; //$NON-NLS-1$
        return toURL(the_spec);
    }

    public static URL toURL( String the_spec ) throws MalformedURLException {
        return new URL(null, the_spec, CorePlugin.RELAXED_HANDLER);
    }

    @Override
    protected String doOtherChecks( Map<String, Serializable> params ) {
        return null;
    }
    
    @Override
    protected DataStoreFactorySpi getDataStoreFactory() {
        return getFactory();
    }

    public String reasonForFailure( URL url ) {
        if( ! isPostGIS(url) )
            return Messages.PostGISServiceExtension_badURL; 
        return reasonForFailure(createParams(url));
    }

}
