package net.refractions.udig.catalog.internal.ui;

import java.io.Serializable;
import java.net.UnknownHostException;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.ui.UDIGConnectionPage;
import net.refractions.udig.catalog.ui.internal.Messages;
import net.refractions.udig.catalog.ui.workflow.ConnectionState;
import net.refractions.udig.catalog.ui.workflow.WorkflowWizardDialog;
import net.refractions.udig.catalog.ui.workflow.WorkflowWizardPage;
import net.refractions.udig.catalog.ui.workflow.Workflow.Listener;
import net.refractions.udig.catalog.ui.workflow.Workflow.State;
import net.refractions.udig.ui.PlatformGIS;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.geotools.data.DataSourceException;

public class ConnectionPage extends WorkflowWizardPage implements UDIGConnectionPage, Listener {

    /** underlying import page * */
    UDIGConnectionPage page;

    public ConnectionPage() {
        super("connection"); // set name later //$NON-NLS-1$
    }

    /**
     * @return The decorated page.
     */
    public UDIGConnectionPage getDecoratedPage() {
        return page;
    }

    public Map<String, Serializable> getParams() {
        return page.getParams();
    }

    @Override
    public void setState( State state ) {
        super.setState(state);

        UDIGConnectionPage tmp;
        
        // create the specific connection page
        tmp = ((ConnectionState) state).getConnectionFactory().createConnectionPage();

        if( tmp!=page && page!=null ){
            page.dispose();
        }
        
        page=tmp;
        
        // do some setup
        page.setWizard(getWizard());

        // we do the instance check to allow the connection page to
        // optionally extend DataPipelinePage.
        if (page instanceof WorkflowWizardPage) {
            ((WorkflowWizardPage) page).setState(state);
        }

        // add a listener to the workflow to determine when the workflow
        // moves back, when this happens, we need to forget about the
        // decorated page
        state.getWorkflow().addListener(this);

    }

    @Override
    public void shown() {
        super.shown();

        if (page != null && page instanceof WorkflowWizardPage) {
            ((WorkflowWizardPage) page).shown();
        }
    }

    public void createControl( Composite parent ) {
        page.createControl(parent);

        setControl(page.getControl());

    }

    @Override
    public void dispose() {
        page.dispose();
        setControl(null);
        page = null;
    }
    
    @Override
    public Control getControl() {
        if( page.getControl()==null || page.getControl().isDisposed() ){
            return null;
        }
        return page.getControl();
    }
    
    @Override
    public boolean isPageComplete() {
        boolean complete = page.isPageComplete();
        if (complete) {
            // set some context for the connection state
            ConnectionState state = (ConnectionState) getState();
            state.setParams(page.getParams());
        }

        return complete;
    }

    @Override
    public String getName() {
        return page.getName();
    }

    @Override
    public Image getImage() {
        return page.getImage();
    }

    @Override
    public String getDescription() {
        return page.getDescription();
    }

    @Override
    public String getTitle() {
        return page.getTitle();
    }

    @Override
    public String getMessage() {
        return page.getMessage();
    }

    @Override
    public void setWizard( IWizard newWizard ) {
        super.setWizard(newWizard);
        if (page != null)
            page.setWizard(newWizard);
    }

    @Override
    public void setVisible( boolean visible ) {
        page.setVisible(visible);
    }

    public void forward( State current, State prev ) {
        // TODO Auto-generated method stub

    }

    /**
     * 52North Changed
     */
    public void backward( State current, State next ) {
        if (next != null && next instanceof ConnectionState) {
            // reset decorated page
            /*
             * page = null; getControl().getDisplay().syncExec( new Runnable() { public void run() {
             * getControl().dispose(); } } ); setControl(null);
             */
        }
    }

    public void statePassed( State state ) {
        PlatformGIS.syncInDisplayThread(new Runnable(){
            public void run() {
                setErrorMessage(null);
            }
        });
    }

    public void stateFailed( State state ) {
        if (state instanceof ConnectionState) {
            ConnectionState connectionState = (ConnectionState) state;
            Map<IService, Throwable> errors = connectionState.getErrors();
            Iterator<Entry<IService, Throwable>> iterator = errors.entrySet().iterator();
            if (iterator.hasNext()) {
                Entry<IService, Throwable> entry = iterator.next();
                Throwable t = entry.getValue();
                final String message = formatException(entry.getKey(), t);
                if (Display.getCurrent() == null) {
                    Display.getDefault().asyncExec(new Runnable(){
                        public void run() {
                            page.setErrorMessage(message);
                        }
                    });
                } else
                    page.setErrorMessage(message);
            }
        }
    }

    private String formatException( IService key, Throwable t ) {
        if (t instanceof UnknownHostException) {
            return key.getIdentifier().getHost() + Messages.ConnectionPage_illegalHost;
        }
        if (t instanceof DataSourceException) {
            String message = t.getMessage();
            if (message.contains("user") && message.contains("does not exist")) //$NON-NLS-1$ //$NON-NLS-2$
                return Messages.ConnectionPage_badUsername;
            if (message.contains("password")) //$NON-NLS-1$
                return Messages.ConnectionPage_badPassword;
        }
        return Messages.ConnectionPage_genericError;
    }

    public void started( State first ) {

    }

    public void finished( State last ) {
        if (!(getContainer() instanceof WorkflowWizardDialog) && getContainer() != null) {
            getWizard().performFinish();
        }
    }

    /**
     * 52North changed Method returns the successor wizard page. If there is no succesor wizard
     * page, null is returned.
     */
    @Override
    public IWizardPage getNextPage() {
        if (!getState().hasNext()) {
            return super.getNextPage();
        }
        return page.getNextPage();
    }

}
