/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.ui;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.internal.ui.ConnectionPage;
import net.refractions.udig.catalog.ui.internal.Messages;
import net.refractions.udig.catalog.ui.workflow.ConnectionState;
import net.refractions.udig.catalog.ui.workflow.Workflow;
import net.refractions.udig.catalog.ui.workflow.WorkflowWizard;
import net.refractions.udig.catalog.ui.workflow.WorkflowWizardDialog;
import net.refractions.udig.catalog.ui.workflow.WorkflowWizardPage;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IPageChangedListener;
import org.eclipse.jface.dialogs.PageChangedEvent;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.IWizardContainer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.PlatformUI;

/**
 * A wizard page that opens a file dialog and closes the wizard when dialog is closed.
 * 
 * @author jeichar
 * @since 0.9.0
 */
public class FileConnectionPage extends WorkflowWizardPage 
	implements UDIGConnectionPage, IPageChangedListener {

    private boolean canFlipToNextPage = false;
    private Set<URL> list;
    private Composite comp;

    private FileConnectionFactory factory = new FileConnectionFactory();
    private FileDialog fileDialog;

    public String getId() {
        return "net.refractions.udig.catalog.ui.openFilePage"; //$NON-NLS-1$
    }

    /**
     * Construct <code>OpenFilePage</code>.
     */
    public FileConnectionPage() {
        super(Messages.OpenFilePage_pageTitle); 
    }

    List<IService> process( List<URL> urls, IProgressMonitor monitor ) {
        List<IService> resources = new ArrayList<IService>();
        monitor.beginTask(Messages.OpenFilePage_1, list.size()); 
        int worked = 0;
        for( URL url : urls ) {
            if (monitor.isCanceled())
                return null;
            try {
                monitor.subTask(url.toExternalForm());
                List<IService> acquire = CatalogPlugin.getDefault().getServiceFactory().acquire(url);
                resources.addAll(acquire);
            } catch (Throwable e) {
                CatalogUIPlugin.log( "error obtaining services from service factory", e); //$NON-NLS-1$
            }
            monitor.worked(worked++);
        }
        return resources;
    }
    
    private void cancelWizard() {
        try {
            getShell().getDisplay().asyncExec(new Runnable(){
                public void run() {
                    findButton(getShell().getChildren(), IDialogConstants.BACK_ID).notifyListeners(
                            SWT.Selection, new Event());
                }

            });
        } catch (Exception e) {
            CatalogUIPlugin.log("", e); //$NON-NLS-1$
        }
    }

    Button findButton( Control[] children, int id ) {
        if (((Integer) getShell().getDefaultButton().getData()).intValue() == id)
            return getShell().getDefaultButton();

        for( Control child : children ) {
            if (child instanceof Button) {
                Button button = (Button) child;
                if (button.getData() != null && ((Integer) button.getData()).intValue() == id)
                    return button;
            }
            if (child instanceof Composite) {
                Composite composite = (Composite) child;
                Button button = findButton(composite.getChildren(), id);
                if (button != null)
                    return button;
            }
        }
        return null;
    }

    Workflow workflow;
    private void finishWizard() {
        try {
        	if (getContainer() instanceof WizardDialog) {
        		final IWizardContainer dialog = getContainer();
        		WorkflowWizard wizard = null;
        		if (dialog instanceof WorkflowWizardDialog) {
        			wizard = ((WorkflowWizardDialog)dialog).getWorkflowWizard();
        		}
        		else if (dialog.getCurrentPage().getWizard() instanceof WorkflowWizard) {
        			wizard = (WorkflowWizard)dialog.getCurrentPage().getWizard();
        		}
        		
        		if (wizard != null) {
        			//move workflow to next state
        			workflow = wizard.getWorkflow();
                    final ConnectionState currentState = (ConnectionState)workflow.getCurrentState();
                    (currentState).setURLs(getURLs());
                    
                    final boolean[] finished=new boolean[]{false};
                    
        			dialog.run(true, true, new IRunnableWithProgress() {
            			public void run(IProgressMonitor monitor) 
            				throws InvocationTargetException ,InterruptedException {
                            monitor.beginTask(Messages.FileConnectionPage_taskname, IProgressMonitor.UNKNOWN); 
                            try {
//                                if (currentState.run(new SubProgressMonitor(monitor, 4)) 
//                                        && hasOneResource(new SubProgressMonitor(monitor, 4),
//                                                currentState.getServices()) ) {
                                    finished[0]=workflow.run(new SubProgressMonitor(monitor, 4));                                    
//                                }else{
//                                    workflow.next(new SubProgressMonitor(monitor, 8));
//                                }
                            }catch (Exception e) {
                                throw new InvocationTargetException(e);
                            }
            			};
            		});	
        			if( finished[0] && dialog!=null && dialog.getShell()!=null && !dialog.getShell().isDisposed()){
                        Display.getDefault().asyncExec(new Runnable(){
                            public void run() {
                                dialog.getShell().close();
                            }
                        });
                    }
        		}
        		
        		//one good hack deserves another
        		if (!(dialog instanceof WizardDialog)) {
        			//manually move wizard to next state
        			((WizardDialog) dialog).addPageChangedListener(this);
        		}
                
        	}
            
        } catch (Exception e) {
            CatalogUIPlugin.log("", e); //$NON-NLS-1$
        }
    }
    
    protected boolean hasOneResource( SubProgressMonitor monitor, List<IService> services ) throws IOException {
        if( services.size()>1 || services.isEmpty() )
            return false;
        
        if (services.get(0).members(monitor).size()==1)
            return true;
        return false;
    }

    public void pageChanged(PageChangedEvent event) {
    	if (getContainer().getCurrentPage() instanceof ConnectionPage) {
    		WizardDialog dialog = (WizardDialog)getContainer();
    		dialog.removePageChangedListener(this);
    		
    		final int button = workflow.hasMoreStates() ? IDialogConstants.NEXT_ID : 
    			IDialogConstants.FINISH_ID;
    		
    		getShell().getDisplay().asyncExec(
    			new Runnable() {
    				public void run() {
    					findButton(getShell().getChildren(), button)
    						.notifyListeners(SWT.Selection,new Event());
    					
    				}
    			}
    		);
    		
    	}
    }

    /**
     * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
     */
    public boolean canFlipToNextPage() {
        return (list != null && list.size() > 1) || canFlipToNextPage;
    }

    /**
     * @see net.refractions.udig.ui.UDIGImportPage#setCanFlipToNextPage(boolean)
     */
    public void setCanFlipToNextPage( boolean value ) {
        canFlipToNextPage = true;
    }

    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl( Composite parent ) {
        comp = new Composite(parent, SWT.NULL);
        // comp.setLayout(new GridLayout(1,true));
        // viewer=new ListViewer(comp);
        // viewer.getControl().setLayoutData(new GridData(SWT.FILL,SWT.FILL,true, true));
        // viewer.setContentProvider(new ArrayContentProvider());
        // viewer.setLabelProvider(new LabelProvider());
        setControl(comp);

        Display.getCurrent().asyncExec(new Runnable(){
            public void run() {
                boolean open = openFileDialog(comp);
                if (open) {
                    finishWizard();
                } else {
                    cancelWizard();
                }                
            }
        });
    }

    private boolean checkDND( FileDialog fileDialog ) {
        try {

            Object context = getState().getWorkflow().getContext();

            // IStructuredSelection selection = ((IDataWizard) getWizard()).getSelection();

            Set<URL> urlList = new HashSet<URL>();

            URL url = factory.createConnectionURL(context);
            if (url != null) {
                urlList.add(url);
            }

            if (urlList.size() != 0) {
                list = urlList;
                String file = urlList.iterator().next().getFile();
                String ext = file.substring(file.lastIndexOf('.'));
                String dir = new File(file).getParent();

                file = file.substring(file.lastIndexOf(File.separator) + 1);
                fileDialog.setFilterPath(dir);
                fileDialog.setFileName(file);

                String[] filters = fileDialog.getFilterExtensions();
                if (filters == null || filters.length == 0) {
                    // no filters set, set em up
                    fileDialog.setFilterExtensions(new String[]{"*" + ext, "*.*"}); //$NON-NLS-1$ //$NON-NLS-2$	
                } else {
                    // we have some filters, look for the one in question
                    // in the list
                    int i = 0;
                    for( ; i < filters.length; i++ ) {
                        if (("*" + ext).equals(filters[i]))break; //$NON-NLS-1$
                    }

                    if (i < filters.length) {
                        // we found it, reorganize the array so that
                        // it is first
                        String[] nfilters = new String[filters.length];
                        nfilters[0] = filters[i];
                        System.arraycopy(filters, 0, nfilters, 1, i);
                        System.arraycopy(filters, i + 1, nfilters, i + 1, filters.length - i - 1);
                        fileDialog.setFilterExtensions(nfilters);
                    } else {
                        // no dice, add the filter
                        String[] nfilters = new String[filters.length + 1];
                        nfilters[0] = "*" + ext; //$NON-NLS-1$
                        System.arraycopy(filters, 0, nfilters, 1, filters.length);
                        fileDialog.setFilterExtensions(nfilters);
                    }
                }

                return true;
            }
        } catch (Exception e) {
            CatalogUIPlugin.log(e.getLocalizedMessage(), e);
        }

        return false;

    }

    public FileDialog getFileDialog() {
        return fileDialog;
    }

    private boolean openFileDialog( Composite parent ) {
        String lastOpenedDirectory = PlatformUI.getPreferenceStore().getString(
                CatalogUIPlugin.PREF_OPEN_DIALOG_DIRECTORY);
        fileDialog = new FileDialog(parent.getShell(), SWT.MULTI);
        List<String> fileTypes = factory.getExtensionList();
       
        StringBuffer all = new StringBuffer();
        for( Iterator<String> i = fileTypes.iterator(); i.hasNext(); ) {
            all.append(i.next());
            if (i.hasNext())
                all.append(";"); //semicolon is magic in eclipse FileDialog
        }
        fileTypes.add(0, all.toString());
        
        fileTypes.add("*.*"); //$NON-NLS-1$

        fileDialog.setFilterExtensions(fileTypes.toArray(new String[fileTypes.size()]));

        if (lastOpenedDirectory != null && !checkDND(fileDialog) ) {
            fileDialog.setFilterPath(lastOpenedDirectory);
        }

        

        // //this is a HACK to check for headless execution
        // if (getContainer() instanceof HeadlessWizardDialog) {
        // if (dnd)
        // return true; //there was a workbench selection that allowed us select a file
        // }
        //        
        String result = fileDialog.open();
        if (result == null) {
            return false;
        }
        String path = fileDialog.getFilterPath();
        PlatformUI.getPreferenceStore().setValue(CatalogUIPlugin.PREF_OPEN_DIALOG_DIRECTORY, path);
        String[] filenames = fileDialog.getFileNames();
        list = new HashSet<URL>();
        for( int i = 0; i < filenames.length; i++ ) {
            try {
                URL url = new File(path + System.getProperty("file.separator") + filenames[i]).toURL(); //$NON-NLS-1$
                list.add(url);
            } catch (Throwable e) {
                CatalogUIPlugin.log("", e); //$NON-NLS-1$
            }
        }
        return true;
        // viewer.setInput(urls);
    }

    public Set<URL> getURLs() {
        if (list == null || list.isEmpty())
            return null;

        return list;
    }

    public Map<String, Serializable> getParams() {
        return null; // no specific connection parameters
    }

}
