/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.catalog.ui.export;

import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.catalog.internal.ui.ImageConstants;
import net.refractions.udig.catalog.internal.ui.Images;
import net.refractions.udig.catalog.ui.ResolveLabelProviderSimple;
import net.refractions.udig.catalog.ui.ResolveTitlesDecorator;
import net.refractions.udig.catalog.ui.internal.Messages;
import net.refractions.udig.catalog.ui.workflow.WorkflowWizardPage;
import net.refractions.udig.ui.CRSDialogCellEditor;

import org.eclipse.jface.dialogs.IPageChangedListener;
import org.eclipse.jface.dialogs.PageChangedEvent;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.geotools.data.FeatureSource;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

/**
 * A page that allows the user to select the layer(s) to save.
 * 
 * @author chorner
 * @since 1.1.0
 */
public class ExportResourceSelectionPage extends WorkflowWizardPage implements IPageChangedListener, ICheckStateListener {
    private CheckboxTreeViewer viewer;

    public ExportResourceSelectionPage( String pageName ) {
        super(pageName);
        setTitle(Messages.LayerSelectionPage_title); 
        setMessage(Messages.LayerSelectionPage_message); 
        setImageDescriptor(Images.getDescriptor(ImageConstants.DATA_WIZBAN));
    }

    public List<Data> getCheckedElements() {
        List<Data> list = new ArrayList<Data>();
        for( Object object : viewer.getCheckedElements() ) {
            Data resource = (Data) object;
            list.add(resource);
        }
        return list;
    }

    public void createControl( Composite parent ) {
        Composite composite = new Composite(parent, SWT.NULL);
        composite.setLayout(new GridLayout());

        viewer = new CheckboxTreeViewer(composite, SWT.CHECK|SWT.SINGLE|SWT.FULL_SELECTION);
        Tree tree = createTree(viewer.getTree());
        
        viewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        viewer.addPostSelectionChangedListener(new ISelectionChangedListener(){

            public void selectionChanged( SelectionChangedEvent event ) {
                getWizard().getContainer().updateButtons();
            }

        });
        viewer.addCheckStateListener(this);
        viewer.setContentProvider(new ResourceContentProvider());

        viewer.setLabelProvider(new ExportResourceLabelProvider());
        viewer.setAutoExpandLevel(3);
        
        final String crs="CRS"; //$NON-NLS-1$
        viewer.setColumnProperties(new String[]{"name",crs}); //$NON-NLS-1$
        viewer.setCellEditors(new CellEditor[]{null, new CRSDialogCellEditor(tree)});
        viewer.setCellModifier(new ICellModifier(){

            public boolean canModify( Object element, String property ) {
                if( crs.equals(property) )
                    return true;
                return false;
            }

            public Object getValue( Object element, String property ) {
                return ((Data)element).getCRS();
            }

            public void modify( Object element, String property, Object value ) {
                TreeItem item=(TreeItem) element;
                Data data=(Data) item.getData();
                data.setCRS((CoordinateReferenceSystem)value);
                viewer.update(data, null);
            }
            
        });
        // use the state to initialize ui
        ExportResourceSelectionState state = (ExportResourceSelectionState) getState();
        setInput(state);

        setControl(composite);
    }
    

    public void checkStateChanged( CheckStateChangedEvent event ) {
        if (event.getChecked()) {
            Object o = event.getElement();
            if (viewer.getGrayed(o)) {
                viewer.setChecked(o, false); //grayed elements cannot be checked
                return; //(this is temporary until multiple format export is supported)
            }
        }
        
        // set all children to same check state
        Object o = event.getElement();
        ResourceContentProvider p = (ResourceContentProvider) viewer.getContentProvider();
        Object[] elements = p.getElements(o);
        if (elements != null && elements.length > 0) {
            for( int i = 0; i < elements.length; i++ )
                viewer.setChecked(elements[i], event.getChecked());
        }

        if (viewer.getCheckedElements().length > 0) { //set complete flag
            setPageComplete(true);
        } else {
            setPageComplete(false);
        }

        syncWithUI(); //update the selected layers value in the state
        
    }


    private Tree createTree( Tree tree ) {
        
        TableLayout tableLayout = new TableLayout();
        tableLayout.addColumnData(new ColumnWeightData(1,true));
        tableLayout.addColumnData(new ColumnWeightData(1,true));
        tree.setLayout(tableLayout);

        TreeColumn name=new TreeColumn(tree, SWT.DEFAULT);
        name.setMoveable(false);
        name.setResizable(true);
        
        TreeColumn projection=new TreeColumn(tree, SWT.DEFAULT);
        projection.setMoveable(false);
        projection.setResizable(true);

        return tree;
    }

    public CheckboxTreeViewer getViewer() {
        return viewer;
    }

    public void syncWithUI() {
        ExportResourceSelectionState state = (ExportResourceSelectionState) getState(); 
        List<Data> selectedLayers = new ArrayList<Data>();
        Object[] elements = getViewer().getCheckedElements();
        for( int i = 0; i < elements.length; i++ ) {
            Data layer = (Data) elements[i];
            selectedLayers.add(layer);
        }
        state.setSelectedLayers(selectedLayers);
    }

    @Override
    public void shown() {
        setInput((ExportResourceSelectionState) getState());
    }

    private void setInput( ExportResourceSelectionState state ) {
        List<Data> resources = state.getLayers();
        viewer.setInput(resources);
        if (resources != null) {
            for( Data data : resources ) {
                viewer.setChecked(data, false);
                if (data.getResource().canResolve(FeatureSource.class))
                    viewer.setGrayed(data, false);
                else
                    viewer.setGrayed(data, true);
            }

            //select possible layers
            for (Data data : resources) {
                if (!viewer.getGrayed(data)) 
                    viewer.setChecked(data, true);
            }
        }
        
        if (viewer.getCheckedElements().length > 0) {
            setPageComplete(true);
        } else {
            setPageComplete(false);
        }
        
        syncWithUI();
    }
    
    /**
     * @see org.eclipse.jface.wizard.WizardPage#isPageComplete()
     */
    @Override
    public boolean isPageComplete() {
        return viewer.getCheckedElements().length > 0;
    }
     
    Button findButton( Control[] children, int id ) {
        if (((Integer) getShell().getDefaultButton().getData()).intValue() == id)
            return getShell().getDefaultButton();

        for( Control child : children ) {
            if (child instanceof Button) {
                Button button = (Button) child;
                if (((Integer) button.getData()).intValue() == id)
                    return button;
            }
            if (child instanceof Composite) {
                Composite composite = (Composite) child;
                Button button = findButton(composite.getChildren(), id);
                if (button != null)
                    return button;
            }
        }
        return null;
    }

    private static class ResourceContentProvider implements ITreeContentProvider {

        @SuppressWarnings("unchecked")
        public Object[] getElements( Object inputElement ) {
            return getChildren(inputElement);
        }

        public void dispose() {
        }

        public void inputChanged( Viewer viewer, Object oldInput, Object newInput ) {
        }

        @SuppressWarnings("unchecked")
        public Object[] getChildren( Object parentElement ) {
            if( parentElement instanceof List ){
                List<Data> resources = (List<Data>) parentElement;
                return resources.toArray();
            }
            return null;
        }

        public Object getParent( Object element ) {
            return null;
        }

        public boolean hasChildren( Object element ) {
            return false;
        }

    }


    /**
     * Adapts the new ResolveTitlesDecorator(new ResolveLabelProviderSimple()) so that it will can appear in the
     * table with the title returned by the decorator as the name column and the projection as the title column
     * @author Jesse
     * @since 1.1.0
     */
    public class ExportResourceLabelProvider implements ITableLabelProvider{

        private DecoratingLabelProvider wrapped;

        public ExportResourceLabelProvider() {
            ResolveTitlesDecorator titleDecorator = new ResolveTitlesDecorator(new ResolveLabelProviderSimple());
            wrapped = new DecoratingLabelProvider(titleDecorator.getSource(),
                    titleDecorator);
        }
        
        public void addListener( ILabelProviderListener listener ) {
            wrapped.addListener(listener);
        }

        public void dispose() {
            wrapped.dispose();
        }

        public boolean isLabelProperty( Object element, String property ) {
            return wrapped.isLabelProperty(element, property);
        }

        public void removeListener( ILabelProviderListener listener ) {
            wrapped.addListener(listener);
        }

        public Image getColumnImage( Object element, int columnIndex ) {
            assert element instanceof Data;
            assert columnIndex < 3;

            Data data=(Data)element;
            switch( columnIndex ) {
            case 0:
                return wrapped.getImage(data.getResource());
            case 1:
                return null;

            default:
                break;
            }
            return null;
        }

        public String getColumnText( Object element, int columnIndex ) {
            assert element instanceof Data;
            assert columnIndex < 3;

            Data data=(Data)element;
            CoordinateReferenceSystem crs = data.getCRS();
			switch( columnIndex ) {
            case 0:
                return wrapped.getText(data.getResource());
            case 1:
            	if( crs==null )
            		return "undefined";
                return crs.getName().toString();

            default:
                break;
            }
            throw new RuntimeException("Don't know how to handle column 3"); //$NON-NLS-1$
        }

    }
    
    public void pageChanged( PageChangedEvent event ) {
    }
}
