package net.refractions.udig.catalog.ui.export;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.ui.internal.Messages;
import net.refractions.udig.catalog.ui.workflow.Workflow;
import net.refractions.udig.core.AdapterUtil;
import net.refractions.udig.ui.ProgressManager;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.PlatformUI;

public class ExportResourceSelectionState extends Workflow.State {

    List<Data> layers;
    List<Data> selectedLayers;
    ISelection selection = null;
    
    /**
     * The selection to save.  Expected value is a map or layers
     * 
     * @param selection
     */
    public ExportResourceSelectionState( ISelection selection ) {
        this.selection=selection;
    }

    public ExportResourceSelectionState() {
        this(null);
    }
    
    public List<Data> getLayers() {
        return layers;
    }
    
    public List<Data> getExportData() {
        return selectedLayers;
    }

    public void setSelectedLayers(List<Data> selectedLayers) {
        this.selectedLayers = selectedLayers; 
    }
    
    @Override
    public void init( IProgressMonitor monitor ) throws IOException {
        super.init(monitor);

        layers = new ArrayList<Data>();
        
        if (selection == null) {
            selection = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getSelection();
        }
        Object items[] = new Object[0];
        if (selection instanceof StructuredSelection) {
            items = ((StructuredSelection) selection).toArray();
        }
        
        for (int i = 0; i < items.length; i++) {
            Object object = items[i];
            convertToGeoResource(object);
        }
    }

    private void convertToGeoResource( Object object ) throws IOException {
        Collection<Data> data = new HashSet<Data>();
        
        tryAdaptingToGeoResource(object, data);
        tryAdaptingToService(object, data);
        
        layers.addAll(data);
    }

    private void tryAdaptingToGeoResource( Object object, Collection<Data> data ) throws IOException {
        if (object instanceof IGeoResource) {
            data.add(new Data((IGeoResource) object));
        } else {
            
            IGeoResource resource = AdapterUtil.instance.getOperationTarget(IGeoResource.class, object, ProgressManager.instance().get());
            if( resource!=null ){
               data.add(new Data(resource)); 
            }
        }
    }

    private void tryAdaptingToService( Object object, Collection<Data> data ) throws IOException {
        if (object instanceof IService) {
            IService service = (IService) object;
            List< ? extends IGeoResource> resources = service.members(ProgressManager.instance().get());
            for( IGeoResource resource : resources ) {
                data.add(new Data(resource));
            }
        } 
        
    }

    @Override
    public boolean run( IProgressMonitor monitor ) throws IOException {
        // complete if all the resources have been "selected"
        if (layers == null || layers.isEmpty() || selectedLayers == null || selectedLayers.isEmpty())
            return false;
        return true;
    }

	@Override
	public String getName() {
		return Messages.CatalogExport_exportLayersTask; 
	}

}