/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog;

import java.io.Serializable;
import java.net.URL;
import java.util.List;
import java.util.Map;

/**
 * Builds service proxies or clones (with an id).
 * <p>
 * Where not specified, sensible defaults will be added to the create options. aka Magic will occur
 * here :-)
 * </p>
 * 
 * @author David Zwiers, Refractions Research
 */
public interface IServiceFactory {

    /**
     * This will create a new IService magically. In some cases sensible default parameters may be
     * added, in addition to parameters removed. An ID will be generated.
     * 
     * @param params
     * @return List<IService>
     */
    List<IService> acquire( Map<String, Serializable> params ); // may look up authentication

    /**
     * This method generates a default set of params, and calls acquire(params).
     * 
     * @param target
     * @return List<IService>
     * @see acquire(params)
     */
    List<IService> acquire( URL target ); // creates a map, may look up authentication

    /**
     * This method is intended to be used when replacing an IService entry in a catalog, or for
     * cloning.
     * 
     * This allows you to retain the URI id, while providing new parameters. This is also
     * intended for persistence frameworks to use. WARNING: This may have undesired
     * results/conflicts when added to a ICatalog if care is not taken when using this method. The ID
     * is usually an exact match to the data, if you are not careful you end up with a IService registered
     * using an ID different from the data it is representing.
     * 
     * @param id
     * @param params
     * @return List<IService>
     */
    List<IService> acquire( URL id, Map<String, Serializable> params ); // may not look up
                                                                        // authentication
    /**
     * @deprecated use {@link #acquire(Map)}
     */
    List<IService> aquire( Map<String, Serializable> params ); // may look up authentication
    /**
     * @deprecated use {@link #acquire(URL)}
     */
    List<IService> aquire( URL target ); // creates a map, may look up authentication
    /**
     * @deprecated use {@link #acquire(URL, Map)}
     */
    List<IService> aquire( URL id, Map<String, Serializable> params ); // may not look up
                                                                        // authentication
}
