package net.refractions.udig.internal.ui;

import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;

/**
 * Public base class for configuring the action bars of a workbench window.
 * <p>
 * An application should declare a subclass of <code>ActionBarAdvisor</code>
 * and override methods to configure a window's action bars to suit the needs of the
 * particular application.
 * </p>
 * <p>
 * The following advisor methods are called at strategic points in the
 * workbench's lifecycle (all occur within the dynamic scope of the call
 * to {@link PlatformUI#createAndRunWorkbench PlatformUI.createAndRunWorkbench}):
 * <ul>
 * <li><code>fillActionBars</code> - called after <code>WorkbenchWindowAdvisor.preWindowOpen</code>
 * to configure a window's action bars</li>
 * </ul>
 * </p>
 * 
 * @see WorkbenchWindowAdvisor#createActionBarAdvisor(IActionBarConfigurer)
 * 
 * @author cole.markham
 * @since 1.0.0
 */
public class UDIGActionBarAdvisor extends ActionBarAdvisor {
    
    /**
     * Default constructor
     * @param configurer
     */
    public UDIGActionBarAdvisor( IActionBarConfigurer configurer ) {
        super(configurer);
    }
    
    @Override
    protected void fillCoolBar( ICoolBarManager coolBar ) {
        IWorkbenchWindow window = getActionBarConfigurer().getWindowConfigurer().getWindow();
        UiPlugin.getDefault().getMenuFactory().fillCoolBar( coolBar, window );
    }
    
    
    @Override
    protected void fillMenuBar( IMenuManager menuBar ) {
        IWorkbenchWindow window = getActionBarConfigurer().getWindowConfigurer().getWindow();
        UiPlugin.getDefault().getMenuFactory().fillMenuBar( menuBar, window );
        
    }

}
