/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.ui;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.geotools.factory.CommonFactoryFinder;
import org.geotools.factory.GeoTools;
import org.geotools.feature.Feature;
import org.opengis.filter.FilterFactory;
import org.opengis.filter.FilterVisitor;
import org.opengis.filter.Id;
import org.opengis.filter.identity.Identifier;

class SelectionFilter implements Id {
    final Collection<String> selectionFids;
	private FilterFactory factory = CommonFactoryFinder.getFilterFactory(GeoTools.getDefaultHints());
    
    public SelectionFilter( Collection<String> fids ) {
        this.selectionFids = fids;
    }

    @Override
    public String toString() {
        if( selectionFids==null )
            return ""; //$NON-NLS-1$
        return selectionFids.toString();
    }

    public Set<Object> getIDs() {
        return new HashSet<Object>( selectionFids );
    }

    public Set<Identifier> getIdentifiers() {
        HashSet<Identifier> identifiers = new HashSet<Identifier>();
        for( String fid : selectionFids ){
            identifiers.add( factory .featureId(fid) );
        }
        return identifiers;
    }

	public boolean evaluate(Object object) {
		if( object instanceof Feature){

	        if( selectionFids==null )
	            return false;
	        Feature feature=(Feature) object;
			return selectionFids.contains(feature.getID());
		}
		return false;
	}

	public Object accept(FilterVisitor visitor, Object extraData) {
		return visitor.visit(this, extraData);
	}

}