/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.printing.ui.internal.editor;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.EventObject;
import java.util.List;

import net.refractions.udig.printing.model.Box;
import net.refractions.udig.printing.model.Page;
import net.refractions.udig.printing.ui.internal.Messages;
import net.refractions.udig.printing.ui.internal.PrintAction;
import net.refractions.udig.printing.ui.internal.PrintingPlugin;
import net.refractions.udig.printing.ui.internal.editor.parts.PartFactory;
import net.refractions.udig.printing.ui.internal.editor.parts.TreePartFactory;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.Project;
import net.refractions.udig.project.internal.ProjectPackage;
import net.refractions.udig.project.internal.ProjectPlugin;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.dnd.TemplateTransferDragSourceListener;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.gef.ui.palette.PaletteViewerProvider;
import org.eclipse.gef.ui.palette.FlyoutPaletteComposite.FlyoutPreferences;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

/**
 * An editor for print pages. Uses GEF to edit the layout of the pages.
 * 
 * @author Richard Gould
 * @since 0.3
 */
public class PageEditor extends GraphicalEditorWithFlyoutPalette implements IAdaptable {

    public final static String EDIT_MAP = "edit map"; //$NON-NLS-1$
    public final static String ID = "net.refractions.udig.printing.ui.internal.editor.pageEditor"; //$NON-NLS-1$

    private boolean savePreviouslyNeeded;
    private Page page;
    private PaletteRoot paletteRoot;
    private PageEditorOutlinePage outlinePage;

    public PageEditor() {
        setEditDomain(new DefaultEditDomain(this));
    }

    protected void configureGraphicalViewer() {
        super.configureGraphicalViewer();

        getGraphicalViewer().setRootEditPart(new ScalableFreeformRootEditPart());

        getGraphicalViewer().setEditPartFactory(new PartFactory());

        ContextMenuProvider provider = new PageContextMenuProvider(getGraphicalViewer(),
                getActionRegistry());
        getGraphicalViewer().setContextMenu(provider);
        getSite().registerContextMenu("net.refractions.udig.printing.editor.contextmenu", //$NON-NLS-1$
                provider, getGraphicalViewer());
    }

    public void commandStackChanged( EventObject event ) {
        if (isDirty()) {
            if (!this.savePreviouslyNeeded) {
                this.savePreviouslyNeeded = true;
                firePropertyChange(IEditorPart.PROP_DIRTY);
            }
        } else {
            savePreviouslyNeeded = false;
            firePropertyChange(IEditorPart.PROP_DIRTY);
        }
        super.commandStackChanged(event);
    }

    protected void setInput( IEditorInput input ) {
        super.setInput(input);

        page = (Page) ((PageEditorInput) input).getProjectElement();

        setPartName(page.getName());
    }

    protected void initializeGraphicalViewer() {
        getGraphicalViewer().setContents(this.page);
    }
    
    @Override
    protected void createGraphicalViewer( Composite parent ) {
        GraphicalViewer viewer = new PrintingScrollingGraphicalViewer(this);
        viewer.createControl(parent);
        setGraphicalViewer(viewer);
        configureGraphicalViewer();
        hookGraphicalViewer();
        initializeGraphicalViewer();
    }
    
    public void doSave( IProgressMonitor monitor ) {
        // TODO fix this at some point, currently doesn't work
        // Platform.run(new SafeRunnable(){
        //
        // public void run() throws Exception {
        // page.getProject().eResource().save(null);
        // }
        //            
        // });
        // setDirty(false);
    }

    public void doSaveAs() {
    }

    /* TODO Permit saving at some point */
    public boolean isDirty() {
        return false;
    }

    /* TODO Permit saving at some point */
    public boolean isSaveAsAllowed() {
        return true;
    }

    
    @SuppressWarnings("unchecked")
    protected void createActions() {
        super.createActions();
        ActionRegistry registry = getActionRegistry();


//        IAction action;
//
//        action = new DirectEditAction((IWorkbenchPart)this);
//        registry.registerAction(action);
//        getSelectionActions().add(action.getId());
//               
//        action = new EditMapAction(this);
//        registry.registerAction(action);
//        getSelectionActions().add(action.getId());
        
        List<BoxAction> actions=PrintingPlugin.getBoxExtensionActions(this);
        
        for( IAction action : actions ) {
            registry.registerAction(action);
            getSelectionActions().add(action.getId());            
        }
        
    }


    protected PaletteRoot getPaletteRoot() {

        if (paletteRoot == null) {
            paletteRoot = PageEditorPaletteFactory.createPalette();
        }

        return paletteRoot;

        // PaletteRoot root = new PaletteRoot();
        //        
        // List catagories = new ArrayList();
        //        
        // PaletteGroup controls = new PaletteGroup("Controls");
        //        
        // ToolEntry tool = new SelectionToolEntry();
        // controls.add(tool);
        // root.setDefaultEntry(tool);
        //        
        // controls.add(new MarqueeToolEntry());
        //        
        // PaletteSeparator separator = new PaletteSeparator(
        // PrintingPlugin.PLUGIN_ID + ".palette.separator");
        //        
        // separator.setUserModificationPermission(
        // PaletteEntry.PERMISSION_NO_MODIFICATION);
        // controls.add(separator);
        //
        // root.addAll(catagories);
        //        
        // return root;
    }

    protected FlyoutPreferences getPalettePreferences() {
        return PageEditorPaletteFactory.createPalettePreferences();
    }

    protected PaletteViewerProvider createPaletteViewerProvider() {
        return new PaletteViewerProvider(getEditDomain()){
            protected void configurePaletteViewer( PaletteViewer viewer ) {
                super.configurePaletteViewer(viewer);
                // create a drag source listener for this palette viewer
                // together with an appropriate transfer drop target listener, this will enable
                // model element creation by dragging a CombinatedTemplateCreationEntries
                // from the palette into the editor
                // @see ShapesEditor#createTransferDropTargetListener()
                viewer.addDragSourceListener(new TemplateTransferDragSourceListener(viewer));
            }
        };
    }

    public void createPartControl( Composite parent ) {
        super.createPartControl(parent);

        // enable printing
        IActionBars actionBars = getEditorSite().getActionBars();
        actionBars.setGlobalActionHandler(ActionFactory.PRINT.getId(), new PrintAction());
    }

    public Object getAdapter( Class type ) {
        if (type == IContentOutlinePage.class) {
            if (outlinePage == null) {
                outlinePage = new PageEditorOutlinePage(this, new TreeViewer());
            }
            return outlinePage;
        }
        if (type.isAssignableFrom(Map.class)) {
        	Map found = null;
        	for (Box box : page.getBoxes()) {
        		if (box instanceof IAdaptable) {
	        		Object obj2 = ((IAdaptable) box).getAdapter(Map.class);
	        		if (obj2 instanceof Map) {
	        			/*
	        			 * If multiple maps are found, return null. This will prevent entities
	        			 * that operate only on one map from losing their context.
	        			 * (Imagine the layers view with two map objects selected. It would 
	        			 * only display the layers from one of the objects - confusing to user)
	        			 */
	        			if (found != null) {
	        				found = null; 
	        				break;
	        			}
	        			found = (Map) obj2;
	        		}
        		}
        	}
        	if (found != null) {
        		return found;
        	}
        }
        return super.getAdapter(type);
    }

    private void disposeOutlinePage() {
        this.outlinePage = null;
    }

    private Page getModel() {
        return page;
    }

    @Override
    public void dispose() {

        if (PlatformUI.getWorkbench().isClosing()) {
            ProjectPlugin.getPlugin().turnOffEvents();

            // save the map's URI in the preferences so that it will be loaded the next time udig is
            // run.

            Resource resource = this.page.eResource();
            if (resource != null) {
                try {
                    IPreferenceStore p = ProjectUIPlugin.getDefault().getPreferenceStore();
                    int numEditors = p.getInt(ID);
                    String id = ID + ":" + numEditors; //$NON-NLS-1$
                    numEditors++;
                    p.setValue(ID, numEditors);
                    String value = resource.getURI().toString();
                    p.setValue(id, value);
                } catch (Exception e) {
                    ProjectUIPlugin.log("Failure saving which maps are open", e); //$NON-NLS-1$
                }
            }
        }

        try {
            getSite().getWorkbenchWindow().run(false, false, new IRunnableWithProgress(){

                public void run( IProgressMonitor monitor ) throws InvocationTargetException,
                        InterruptedException {
                    Project p = page.getProjectInternal();
                    try {
                        if (p != null) {
                            if (p.eResource() != null && p.eResource().isModified()) {
                                p.eResource().save(ProjectPlugin.getPlugin().saveOptions);
                            }
                            saveAndUnload();
                        } else {
                            saveAndUnload();
                        }
                    } catch (IOException e) {
                        ProjectPlugin.log("", e); 
                    }

                    // need to kick the Project so viewers will update
                    p.eNotify(new ENotificationImpl((InternalEObject) p, Notification.SET,
                            ProjectPackage.PROJECT__ELEMENTS_INTERNAL, null, null));
                }

                private void saveAndUnload() throws IOException {
                    final Resource resource = page.eResource();

                    resource.save(ProjectPlugin.getPlugin().saveOptions);
                    if (!resource.getContents().contains(
                            ProjectPlugin.getPlugin().getProjectRegistry()))
                        ;
                    resource.unload();
                }
            });
        } catch (Exception e) {
            ProjectPlugin.log("", e); 
        }
        super.dispose();

    }
    public class PageEditorOutlinePage extends ContentOutlinePage {

        private PageEditor editor;
        private PageBook pagebook;

        /**
         * Construct <code>PageEditorOutlinePage</code>.
         * 
         * @param viewer
         */
        public PageEditorOutlinePage( PageEditor editor, EditPartViewer viewer ) {
            super(viewer);

            if (editor == null) {
                throw new IllegalArgumentException(Messages.PageEditor_error_nullEditor); 
            }
            this.editor = editor;
        }

        /**
         * TODO summary sentence for createControl ...
         * 
         * @see org.eclipse.ui.part.IPage#createControl(org.eclipse.swt.widgets.Composite)
         * @param parent
         */
        public void createControl( Composite parent ) {
            pagebook = new PageBook(parent, SWT.NONE);
            Control outline = getViewer().createControl(pagebook);
            getViewer().setEditDomain(editor.getEditDomain());
            getViewer().setEditPartFactory(new TreePartFactory());
            // getViewer().setKeyHandler(editor.getCommonKeyHandler());

            ContextMenuProvider cmProvider = new PageContextMenuProvider(getViewer(), editor
                    .getActionRegistry());
            getViewer().setContextMenu(cmProvider);
            getSite().registerContextMenu("net.refractions.udig.printing.outline.contextmenu", //$NON-NLS-1$
                    cmProvider, getSite().getSelectionProvider());
            editor.getSelectionSynchronizer().addViewer(getViewer());
            getViewer().setContents(editor.getModel());
            pagebook.showPage(outline);

        }
        /**
         * TODO summary sentence for getControl ...
         * 
         * @see org.eclipse.ui.part.IPage#getControl()
         * @return
         */
        public Control getControl() {
            return pagebook;
        }
        /**
         * TODO summary sentence for dispose ...
         * 
         * @see org.eclipse.ui.part.IPage#dispose()
         */
        public void dispose() {
            editor.getSelectionSynchronizer().removeViewer(getViewer());
            super.dispose();
            editor.disposeOutlinePage();
        }
    }
}