/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.printing.ui.internal.editor;

import net.refractions.udig.printing.model.Box;
import net.refractions.udig.printing.model.impl.LabelBoxPrinter;
import net.refractions.udig.printing.ui.IBoxEditAction;
import net.refractions.udig.printing.ui.internal.editor.figures.BoxFigure;
import net.refractions.udig.printing.ui.internal.editor.parts.BoxPart;
import net.refractions.udig.printing.ui.internal.editor.parts.LabelCellEditorLocator;
import net.refractions.udig.printing.ui.internal.editor.parts.LabelDirectEditManager;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.widgets.Text;

/**
 * Renames a LabelBox
 * 
 * @author Jesse
 * @since 1.1.0
 */
public class RenameLabelBox implements IBoxEditAction {

    private BoxPart owner;
    private LabelDirectEditManager manager;
    private String value;
    private boolean disposed=false;
	private LabelBoxPrinter labelBox;

    public void perform() {
        value=null;
        disposed=false;
        labelBox = (LabelBoxPrinter) ((Box) owner.getModel()).getBoxPrinter();
        if (manager == null) {
            BoxFigure nodeFigure = (BoxFigure) owner.getFigure();
            manager = new LabelDirectEditManager(owner, TextCellEditor.class,
                    new LabelCellEditorLocator(nodeFigure), nodeFigure){

                private boolean committing;
                
                @Override
				protected void initCellEditor() {
					super.initCellEditor();
					Text text = (Text) getCellEditor().getControl();
					text.setText(labelBox.getText());
				}

                @Override
                protected void commit() {
                    if (committing)
                        return;
                    committing = true;
                    try {
                        eraseFeedback();
                        value=(String)getCellEditor().getValue();
                    } finally {
                        bringDown();
                        committing = false;
                    }
 
                }
                @Override
                protected void bringDown() {
                    super.bringDown();
                    disposed=true;
                }
            };
        }
        
        
        manager.show();
    }

    public Command getCommand() {
        if( value==null )
            return null;
        return new Command(){

            private String oldValue;
            private String newValue;

            @Override
            public boolean canExecute() {
                return isDone();
            }

            @Override
            public void execute() {
                oldValue = labelBox.getText();
                newValue = value;
                labelBox.setText(newValue);
            }

            @Override
            public void redo() {
                labelBox.setText(newValue);
            }

            @Override
            public void undo() {
                labelBox.setText(oldValue);
            }
        };
    }

    public void init( BoxPart owner ) {
        this.owner = owner;
    }

    public boolean isDone() {
        return disposed;
    }

}
