package net.refractions.udig.catalog.ui;

import net.refractions.udig.catalog.ui.ConnectionFactoryManager.Descriptor;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * Descriptor for net.refractions.udig.catalog.ui.connectionFactory extensions.
 * 
 * @author Justin Deoliveira,Refractions Research Inc.,jdeolive@refractions.net
 *
 */
public class UDIGConnectionFactoryDescriptor {
	UDIGConnectionFactory factory;
	private Descriptor<UDIGConnectionPage> wizardPage;
	
	public UDIGConnectionFactoryDescriptor(Descriptor<UDIGConnectionFactory> factoryDescriptor) 
		throws CoreException {
		factory=factoryDescriptor.getConcreteInstance();
			factory.setDescriptor(this);
			wizardPage = ConnectionFactoryManager.instance().getPageDescriptor(factoryDescriptor);
	}
	
	public UDIGConnectionFactory getConnectionFactory() {
		return factory;
	}
	
	public UDIGConnectionPage createConnectionPage() throws CoreException {
		UDIGConnectionPage page = 
			wizardPage.getConcreteInstance();
			
		page.setTitle(getLabel());
		page.setDescription(getDescription());
		page.setImageDescriptor(getDescriptionImage());
		
		return page;
	}

	public String getLabel() {
		return wizardPage.getConfigurationElement().getAttribute("name"); //$NON-NLS-1$
	}
	
	public String getDescription() {
		String desc = wizardPage.getConfigurationElement().getAttribute("description"); //$NON-NLS-1$
		if (desc == null)
			return ""; //$NON-NLS-1$
		
		return desc.trim();
	}
	
	public String getId() {
		return wizardPage.getConfigurationElement().getAttribute("id"); //$NON-NLS-1$
	}
	
	public ImageDescriptor getImage() {
		String ns = wizardPage.getConfigurationElement().getNamespace();
		String banner = wizardPage.getConfigurationElement().getAttribute("icon"); //$NON-NLS-1$
		
		if (banner == null)
			return null;
		
		return AbstractUIPlugin.imageDescriptorFromPlugin(ns,banner);
	}
	
	public ImageDescriptor getDescriptionImage() {
		String ns = wizardPage.getConfigurationElement().getNamespace();
		String banner = wizardPage.getConfigurationElement().getAttribute("banner"); //$NON-NLS-1$
		
		if (banner == null)
			return null;
		
		return AbstractUIPlugin.imageDescriptorFromPlugin(ns,banner);
	}
	
	public String getServiceType() {
		return wizardPage.getConfigurationElement().getAttribute("type"); //$NON-NLS-1$
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) 
			return true;
		
		if (obj instanceof UDIGConnectionFactoryDescriptor) {
			UDIGConnectionFactoryDescriptor descriptor = 
				(UDIGConnectionFactoryDescriptor)obj;
	
			return getId() != null && 
				getId().equals(descriptor.getId());
		}
		
		return false;
	}
	
	@Override
	public int hashCode() {
		if (getId() == null) 
			return "".hashCode(); //$NON-NLS-1$
		return getId().hashCode();
	}
}