package net.refractions.udig.internal.ui;


import net.refractions.udig.ui.UIUtilities;
import net.refractions.udig.ui.preferences.PreferenceConstants;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IconAndMessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.accessibility.AccessibleAdapter;
import org.eclipse.swt.accessibility.AccessibleEvent;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

public class LinuxAdvancedDialog extends IconAndMessageDialog {

	private StyledText styledMessage;
	private Link linkedMessage;
	private String url = "http://jira.codehaus.org/browse/UDIG-1110";

	public LinuxAdvancedDialog(Shell parentShell) {
		super(parentShell);
		setShellStyle(getShellStyle() | SWT.RESIZE);
		this.message = "Some Linux users have experienced issues with map display being slow or strange artifacts on some versions of Linux. "+
					   "This is a bug with SWT. For more information please see JIRA bug " +
					   "<a href=\""+url+"\">UDIG-1110</a>. There is a "+
					   "link there to <a href=\"https://bugs.eclipse.org/bugs/show_bug.cgi?id=163069\">the SWT bug</a>. Please vote for it!\n\n"+
					   "A workaround is available by disabling advanced graphics. You may do so now, or you may "+
					   "toggle this later under preferences.\n\n" +
					   "Advanced graphics are used in the interface itself, such as the zoom tool for example.";
	}

	@Override
	protected Image getImage() {
		return getWarningImage();
	}

   protected Control createMessageArea(Composite composite) {
        // create composite
        // create image
        Image image = getImage();
        if (image != null) {
            imageLabel = new Label(composite, SWT.NULL);
            image.setBackground(imageLabel.getBackground());
            imageLabel.setImage(image);
            addAccessibleListeners(imageLabel,image);
            imageLabel.setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_BEGINNING));
        }
        // create message
        if (message != null) {
        	linkedMessage = new Link(composite, getMessageLabelStyle());
//            styledMessage = new StyledText(composite, getLinMessageLabelStyle());
        	linkedMessage.setText(message);
        	linkedMessage.setToolTipText("Open in web browser: "+url);
            GridData data = new GridData(GridData.GRAB_HORIZONTAL
                    | GridData.HORIZONTAL_ALIGN_FILL
                    | GridData.VERTICAL_ALIGN_BEGINNING);
            data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
            linkedMessage.setLayoutData(data);
            linkedMessage.addListener(SWT.Selection, new Listener() {
			
				public void handleEvent(Event event) {
					UIUtilities.openLink(event.text);
				}
			
			});
        }
        return composite;
    }
   
   private void addAccessibleListeners(Label label, final Image image) {
		label.getAccessible().addAccessibleListener(
				new AccessibleAdapter(){
    				public void getName(AccessibleEvent event) {
    					final String accessibleMessage = getAccessibleMessageFor(image);
    					if(accessibleMessage == null) {
							return;
						}    					
    					event.result = accessibleMessage;
    				}
    			}
		);
	}
	
    private String getAccessibleMessageFor(Image image){
    	if(image.equals(getErrorImage())) {
			return JFaceResources.getString("error");//$NON-NLS-1$
		}
    	
    	if(image.equals(getWarningImage())) {
			return JFaceResources.getString("warning");//$NON-NLS-1$
		}
    	
    	if(image.equals(getInfoImage())) {
			return JFaceResources.getString("info");//$NON-NLS-1$
		}
    	
    	if(image.equals(getQuestionImage())) {
			return JFaceResources.getString("question"); //$NON-NLS-1$
		}
    	
    	return null;
    }
	
	@Override
	protected Control createDialogArea(Composite parent) {
        
        
        Composite composite = (Composite) super.createDialogArea(parent);
        ((GridLayout)composite.getLayout()).numColumns = 2;
        ((GridLayout)composite.getLayout()).makeColumnsEqualWidth = false;
        
        createMessageArea(composite);
        
        return composite;
	}
	
	
	protected void createButtonsForButtonBar(Composite parent) {
		// create OK and Cancel buttons by default
		Button b = createButton(parent, IDialogConstants.OK_ID, "Disable Advanced Graphics",
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				"Do Nothing", false);
        b.setFocus();
	}

	protected void configureShell( Shell newShell ) {
        newShell.setText("Possible Linux Issue"); 
        newShell.setImage(UiPlugin.getDefault().create("icon32.gif").createImage()); //$NON-NLS-1$
    }

	protected void okPressed() {
    	IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();
    	boolean useAdvancedGraphics = store.getBoolean(net.refractions.udig.ui.preferences.PreferenceConstants.P_ADVANCED_GRAPHICS); 
    	
    	if (useAdvancedGraphics) {
    		store.setValue(PreferenceConstants.P_ADVANCED_GRAPHICS, false);
    	}
    	
        super.okPressed();
    }
	
	
}