/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.mapgraphic.internal;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.ui.ResolveLabelProviderSimple;
import net.refractions.udig.catalog.ui.ResolveTitlesDecorator;
import net.refractions.udig.mapgraphic.MapGraphicPlugin;
import net.refractions.udig.project.ui.ApplicationGIS;
import net.refractions.udig.ui.ProgressManager;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

/**
 * Opens a dialog with all the MapGraphics
 * 
 * @author Jesse
 * @since 1.1.0
 */
public class OtherAction implements IWorkbenchWindowActionDelegate, ITreeContentProvider {

    public void dispose() {
    }

    public void init( IWorkbenchWindow window ) {
    }

    public void run( IAction action ) {
        TitleAreaDialog d = new TitleAreaDialog(Display.getCurrent().getActiveShell()){
            private TreeViewer viewer;
            @Override
            protected int getShellStyle() {
                return SWT.RESIZE|SWT.MAX|SWT.CLOSE|SWT.MIN|SWT.APPLICATION_MODAL;
            }
            
            @Override
            protected void configureShell( Shell newShell ) {
                newShell.setText(Messages.OtherAction_shellText);
                super.configureShell(newShell);
            }
            
            @Override
            protected Point getInitialSize() {
                return new Point(400,400);
            }
            
            @Override
            protected Control createContents( Composite parent ) {
                Control control = super.createContents(parent);
                setTitle(Messages.OtherAction_wizardTitle);
                setMessage(Messages.OtherAction_message1);
                return control;
            }
            
            @Override
            protected Control createDialogArea( Composite parent ) {
                Composite composite = (Composite) super.createDialogArea(parent);
                org.eclipse.swt.widgets.Tree tree=new org.eclipse.swt.widgets.Tree(composite, SWT.V_SCROLL|SWT.MULTI);
                viewer = new TreeViewer(tree);

                GridData gridData = new GridData(GridData.FILL_BOTH);
                gridData.heightHint=SWT.DEFAULT;
                gridData.widthHint=SWT.DEFAULT;
                gridData.verticalSpan=4;
                tree.setLayoutData(gridData);
                viewer.setContentProvider(OtherAction.this);
                ResolveLabelProviderSimple resolveLabelProviderSimple = new ResolveLabelProviderSimple();
                ResolveTitlesDecorator resolveTitlesDecorator = new ResolveTitlesDecorator(
                        resolveLabelProviderSimple, true);
                viewer.setLabelProvider(new DecoratingLabelProvider(resolveLabelProviderSimple,
                        resolveTitlesDecorator));
                viewer.setInput(CatalogPlugin.getDefault().getLocalCatalog().getById(
                        IService.class, MapGraphicService.SERVICE_URL,
                        ProgressManager.instance().get()));
                viewer.addPostSelectionChangedListener(new ISelectionChangedListener(){

                    public void selectionChanged( SelectionChangedEvent event ) {
                        IStructuredSelection s=(IStructuredSelection) event.getSelection();
                        if( s.isEmpty() )
                            return; 
                        
                        String title=null;
                        try {
                            title = ((IGeoResource)s.getFirstElement()).getInfo(null).getTitle();
                        } catch (IOException e) {
                            MapGraphicPlugin.log("", e); //$NON-NLS-1$
                        }
                        if( s.size()==1 && title !=null ){
                            setMessage(Messages.OtherAction_p1+title+Messages.OtherAction_p2);    
                        }else{
                            setMessage(Messages.OtherAction_addAll);
                        }
                    }
                    
                });
                viewer.addDoubleClickListener(new IDoubleClickListener(){

                    public void doubleClick( DoubleClickEvent event ) {
                        buttonPressed(IDialogConstants.OK_ID);
                    }

                });
                return composite;
            }

            @Override
            protected void createButtonsForButtonBar( Composite parent ) {
                createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
                createButton(parent, IDialogConstants.OK_ID, Messages.OtherAction_addButton, true);
            }
            
            @Override
            protected void buttonPressed( int buttonId ) {
                if (buttonId == IDialogConstants.OK_ID) {
                    IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
                    Iterator iter = selection.iterator();
                    List<IGeoResource> resourceList = new ArrayList<IGeoResource>();
                    while( iter.hasNext() )
                        resourceList.add((IGeoResource) iter.next());
                    ApplicationGIS.addLayersToMap(null, resourceList, -1);
                }
                super.buttonPressed(buttonId);
            }
        };
        d.open();

    }

    public void selectionChanged( IAction action, ISelection selection ) {
    }

    public Object[] getElements( Object inputElement ) {
        if (inputElement instanceof IService) {
            try {
                return ((IService) inputElement).members(ProgressManager.instance().get())
                        .toArray();
            } catch (IOException e) {
                throw (RuntimeException) new RuntimeException().initCause(e);
            }
        }
        return null;
    }

    public void inputChanged( Viewer viewer, Object oldInput, Object newInput ) {
    }

    public Object[] getChildren( Object parentElement ) {
        return getElements(parentElement);
    }

    public Object getParent( Object element ) {
        return null;
    }

    public boolean hasChildren( Object element ) {

        if (element instanceof IService) {
            return true;
        }
        return false;
    }

}
