/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.printing.model.impl;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Iterator;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import net.refractions.udig.printing.model.AbstractBoxPrinter;
import net.refractions.udig.printing.model.PrintingModelPlugin;
import net.refractions.udig.printing.model.internal.Messages;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.IMap;
import net.refractions.udig.project.IMapCompositionListener;
import net.refractions.udig.project.IMapListener;
import net.refractions.udig.project.IProject;
import net.refractions.udig.project.MapCompositionEvent;
import net.refractions.udig.project.MapEvent;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.render.CompositeRenderContext;
import net.refractions.udig.project.internal.render.RenderContext;
import net.refractions.udig.project.internal.render.RenderFactory;
import net.refractions.udig.project.internal.render.RenderManager;
import net.refractions.udig.project.internal.render.Renderer;
import net.refractions.udig.project.internal.render.RendererCreator;
import net.refractions.udig.project.internal.render.ViewportModel;
import net.refractions.udig.project.internal.render.impl.RenderContextImpl;
import net.refractions.udig.project.internal.render.impl.RendererCreatorImpl;
import net.refractions.udig.project.render.IViewportModel;
import net.refractions.udig.project.render.IViewportModelListener;
import net.refractions.udig.project.render.RenderException;
import net.refractions.udig.project.render.ViewportModelEvent;
import net.refractions.udig.project.render.displayAdapter.IMapDisplay;
import net.refractions.udig.project.render.displayAdapter.MapDisplayEvent;
import net.refractions.udig.project.ui.ApplicationGIS;
import net.refractions.udig.project.ui.ApplicationGIS.DrawMapParameter;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IMemento;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import com.vividsolutions.jts.geom.Envelope;

/**
 * Box Printer for MapBox objects.
 * 
 * @author Jesse
 * @since 1.1.0
 */
public class MapBoxPrinter extends AbstractBoxPrinter implements IAdaptable {
    Map map;
    

    IMapCompositionListener layerListener=new IMapCompositionListener(){

        public void changed( MapCompositionEvent event ) {
            getBox().notifyPropertyChange(new PropertyChangeEvent(MapBoxPrinter.this, "layers", event.getOldValue(), event.getNewValue())); //$NON-NLS-1$
        }
        
    };
    
    IViewportModelListener viewportListener=new IViewportModelListener(){

        public void changed( ViewportModelEvent event ) {
            getBox().notifyPropertyChange(new PropertyChangeEvent(MapBoxPrinter.this, "viewport", event.getOldValue(), event.getNewValue())); //$NON-NLS-1$
        }
    };
    
    IMapListener mapListener=new IMapListener(){

        public void changed( MapEvent event ) {
            switch( event.getType() ) {
            case VIEWPORT_MODEL:
                if( event.getNewValue()!=null ){
                    ((ViewportModel)event.getNewValue()).addViewportModelListener(viewportListener);
                }
                if( event.getOldValue()!=null ){
                    ((ViewportModel)event.getOldValue()).removeViewportModelListener(viewportListener);
                }
                break;

            default:
                break;
            }
        }
        
    };
    
    public Map getMap() {
        return map;
    }
    
    @SuppressWarnings("unchecked")
    public void setMap( IMap map2 ) {
        Map oldMap=map;
        if( oldMap!=null ){
            oldMap.removeMapCompositionListener(layerListener);
            oldMap.getViewportModel().removeViewportModelListener(viewportListener);
            oldMap.removeMapListener(mapListener);
        }
        this.map = (Map) map2;
        if( map!=null ){
            map.addMapCompositionListener(layerListener);
            map.getViewportModel().addViewportModelListener(viewportListener);
            map.addMapListener(mapListener);
        }
    }

    public void draw( Graphics2D graphics, IProgressMonitor monitor ) {
        try{
            Dimension size = this.getBox().getSize();
            ApplicationGIS.drawMap(new DrawMapParameter(graphics, new java.awt.Dimension(size.width, size.height), getMap(), monitor));
        } catch (RenderException e) {
            PrintingModelPlugin.log( null, e);
        }
    }

    BufferedImage preview;
	State current;
	public void createPreview(Graphics2D graphics,
			final IProgressMonitor monitor) {
		Dimension size = getBox().getSize();

        try {
            if(map != null){
                preview = new BufferedImage(size.width, size.height,
                        BufferedImage.TYPE_INT_ARGB);
                Graphics2D g = preview.createGraphics();
                try{
                ApplicationGIS.drawMap(new DrawMapParameter(g, new java.awt.Dimension(size.width, size.height), this.map, monitor));
                }finally{
                    g.dispose();
                }
                current = new State(map.getViewportModel(), getBox().getSize());
                graphics.drawImage(preview, 0, 0, size.width, size.height, 0, 0,
                        preview.getWidth(), preview.getHeight(), null);
            } 
        }catch (Exception e) {
			PrintingModelPlugin.log("", e); //$NON-NLS-1$
			String message = "Error rendering Map:  " + e.getMessage(); //$NON-NLS-1$
			drawErrorMessage(graphics, message);
		}
	}
    
    
	private void drawErrorMessage(Graphics2D g, String message) {
		Dimension size = getBox().getSize();
		g.setColor(Color.YELLOW);
		Rectangle2D bounds = g.getFontMetrics().getStringBounds(message, g);
		g.fillRect( (int)(size.width/2-bounds.getWidth()/2)-2, (int)(size.height/2-bounds.getHeight()/2)+2, 
		        (int)bounds.getWidth()+4, (int)bounds.getHeight()+4);
		g.setColor(Color.BLACK);
		g.drawString(message, (int)(size.width/2-bounds.getWidth()/2), (int)(size.height/2+bounds.getHeight()/2) );
	}
    
    static class State{
        private double aspectRatio;
        public State( IViewportModel viewportModel, Dimension size ) {
            bounds=viewportModel.getBounds();
            crs=viewportModel.getCRS();
            this.aspectRatio=(double)size.height/(double)size.width;
        }
        Envelope bounds;
        CoordinateReferenceSystem crs;
    }

    public boolean isNewPreviewNeeded() {
        final Dimension size=getBox().getSize();
        double aspectRatio=(double)size.height/(double)size.width;
        return preview==null || current==null || 
        !current.bounds.equals(map.getViewportModel().getBounds()) ||
        current.crs!=map.getViewportModel().getCRS() ||
        Math.abs(current.aspectRatio-aspectRatio)>0.00001 ;
    }
    
    public void save(IMemento memento) {
        System.out.println(map.getID());
        memento.putString("mapId", map.getID().toString()); //$NON-NLS-1$
        memento.putString("projectId", map.getProject().getID().toString()); //$NON-NLS-1$
    }
    
    public void load( IMemento value ) {
            URI mapId = URI.createURI(value.getString("mapId")); //$NON-NLS-1$
            URI projectId = URI.createURI(value.getString("projectId")); //$NON-NLS-1$
            
            List< ? extends IProject> projects = ApplicationGIS.getProjects();
            
            for( IProject project : projects ) {
                if( project.getID().equals(projectId) ){
                    List<IMap> maps = project.getElements(IMap.class);
                    for( IMap map : maps ) {
                        if( map.getID().equals(mapId) ){
                            setMap(map);
                            break;
                        }
                    }
                }
            }
    }

    public String getExtensionPointID() {
        return "net.refractions.udig.printing.ui.standardBoxes"; //$NON-NLS-1$
    }

	@SuppressWarnings("unchecked")
    public Object getAdapter(Class adapter) {
		if (adapter.isAssignableFrom(Map.class)) {
			return this.map;
		}
		return null;
	}


}
