/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.printing.ui.internal.editor;

import java.util.concurrent.atomic.AtomicInteger;

import net.refractions.udig.printing.model.BoxPrinter;
import net.refractions.udig.printing.ui.IBoxEditAction;
import net.refractions.udig.printing.ui.internal.PrintingPlugin;
import net.refractions.udig.printing.ui.internal.editor.parts.PrintingEditPolicy;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;

/**
 * An action that is added to a PageEditor. Each BoxAction corresponds to a editAction in a box
 * extension point.
 * 
 * @author Jesse
 * @since 1.1.0
 */
public class BoxAction extends SelectionAction {

    private IConfigurationElement editActionElement;
    Request request;
    private String acceptable;
    private Class<? extends BoxPrinter> acceptableClass;
    private IBoxEditAction editAction;
    private static AtomicInteger nextID = new AtomicInteger(0);

    public BoxAction( IWorkbenchPart part, IConfigurationElement editActionElement2,
            String acceptable ) {
        super(part);
        this.editActionElement = editActionElement2;
        final int id = nextID.getAndIncrement();
        this.acceptable = acceptable;

        setText(editActionElement2.getAttribute("name")); //$NON-NLS-1$
        String attribute = editActionElement2.getAttribute("image");//$NON-NLS-1$
        ImageDescriptor image; 
        if( attribute!=null && attribute.trim().length()!=0){
            image = PrintingPlugin.imageDescriptorFromPlugin(editActionElement2
                    .getNamespace(), attribute);
            setImageDescriptor(image);
        }
        setId("BOX_EDIT_ACTION_" + getText() + id); //$NON-NLS-1$

        request = new Request(){
            String type=getText() + "_REQUEST_" + id; //$NON-NLS-1$
            @Override
            public Object getType() {
                return type;
            }
        }; 
    }

    public BoxAction( BoxAction action ) {
        super(action.getWorkbenchPart());
        setText(action.getText());
        setToolTipText(action.getToolTipText());
        setId(action.getId());
        setActionDefinitionId(action.getActionDefinitionId());
        this.acceptable=action.acceptable;
        this.acceptableClass=action.acceptableClass;
        this.editActionElement=action.editActionElement;
        this.request=action.request;
    }
    
    @Override
    protected boolean calculateEnabled() {
        if (getSelectedObjects().size() == 1 && (getSelectedObjects().get(0) instanceof EditPart)) {
            EditPart part = (EditPart) getSelectedObjects().get(0);
            return part.understandsRequest(request);
        }
        return false;
    }

    @Override
    public void run() {
        try {
            EditPart part = (EditPart) getSelectedObjects().get(0);
            part.performRequest(request);
        } catch (ClassCastException e) {
            PrintingPlugin.log("", e); //$NON-NLS-1$
        } catch (IndexOutOfBoundsException e) {
            PrintingPlugin.log("", e); //$NON-NLS-1$
        }
    }

    @SuppressWarnings("unchecked")
    public synchronized Class< ? extends BoxPrinter> getAcceptablePrinterClass( Class< ? extends BoxPrinter> class1 ) {
        if (acceptableClass == null) {
            try {
                acceptableClass = (Class< ? extends BoxPrinter>) Class.forName(acceptable, true, class1.getClassLoader());
            } catch (Exception e) {
                return null;
            }
        }
        return acceptableClass;
    }

    public synchronized EditPolicy getEditPolicy() {
        PrintingEditPolicy editPolicy = new PrintingEditPolicy(new BoxAction(this));
        return editPolicy;
    }

    public Request getRequest() {
        return request;
    }

    public synchronized IBoxEditAction getBoxEditAction() {
        if( editAction==null ){
            try{
                editAction=(IBoxEditAction) editActionElement.createExecutableExtension("class"); //$NON-NLS-1$
            }catch (Exception e) {
                PrintingPlugin.getDefault().getLog().log(new Status(IStatus.ERROR, 
                        editActionElement.getNamespace(), IStatus.OK, "", e)); //$NON-NLS-1$
            }
        }
        return editAction;
    }
    
}
