/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.internal.commands;

import net.refractions.udig.project.command.AbstractCommand;
import net.refractions.udig.project.command.UndoableMapCommand;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.Messages;

import org.eclipse.core.runtime.IProgressMonitor;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;

/**
 * Change the CRS of a map.
 * 
 * @author Jesse
 * @since 1.0.0
 */
public class ChangeCRSCommand extends AbstractCommand implements UndoableMapCommand {
    private static final String NAME=Messages.ChangeCRSCommand_name; 
    private Map map;
    private CoordinateReferenceSystem crs;
    private CoordinateReferenceSystem oldCRS;
    private double height;
    private double width;
    private Envelope bounds;
    private Coordinate center;
    /**
     * Construct <code>AddLayerCommand</code>.
     * 
     * @param layer the layer that will be added.
     */
    public ChangeCRSCommand( Map map, CoordinateReferenceSystem crs ) {
        this.map = map;
        this.crs = crs;
        this.oldCRS = map.getViewportModelInternal().getCRS();
        this.height = map.getViewportModelInternal().getHeight();
        this.width = map.getViewportModelInternal().getWidth();
        this.bounds = map.getViewportModelInternal().getBounds();
        this.center = map.getViewportModelInternal().getCenter();
    }

    /**
     * Remove the layer that was added during execution.
     * 
     * @see net.refractions.udig.project.command.UndoableCommand#rollback()
     */
    public void rollback( IProgressMonitor monitor ) throws Exception {
        monitor.beginTask(Messages.ChangeCRSCommand_undoName+NAME, 1); 
        map.getViewportModelInternal().setCRS(oldCRS);
        map.getViewportModelInternal().setCenter(center);
        map.getViewportModelInternal().setHeight(height);
        map.getViewportModelInternal().setWidth(width);
        map.getViewportModelInternal().setBounds(bounds);
        monitor.done();
    }

    /**
     * Changes the maps' CRS. Defensive programming is recommended but command framework protects
     * against exceptions raised in commands.
     * 
     * @see net.refractions.udig.project.command.MapCommand#run()
     */
    public void run( IProgressMonitor monitor ) throws Exception {
        monitor.beginTask(NAME, 1);
        map.getViewportModelInternal().setCRS(crs);
        monitor.done();
    }

    /**
     * Each command has a name that is displayed with the undo/redo buttons.
     * 
     * @see net.refractions.udig.project.command.MapCommand#getName()
     */
    public String getName() {
        return NAME;
    }

}