/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.render;

import java.awt.Point;
import java.awt.geom.AffineTransform;

import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.render.impl.ViewportModelImpl;
import net.refractions.udig.project.render.IViewportModel;
import net.refractions.udig.project.render.displayAdapter.IMapDisplayListener;

import org.eclipse.emf.ecore.EObject;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;

/**
 * TODO Purpose of net.refractions.udig.project.internal.render
 * <p>
 * </p>
 * 
 * @author Jesse
 * @since 1.0.0
 * @model
 */
public interface ViewportModel extends EObject, IMapDisplayListener, IViewportModel {

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    String copyright = "uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details."; //$NON-NLS-1$

    /**
     * A Default setting to use for the viewport screen size.
     * 
     * @model
     */
    public static final Envelope NIL_BBOX = new Envelope(0, 0, 0, 0);

    /**
     * A Default setting to use for the viewport CRS.
     * 
     * @model
     */
    public static final CoordinateReferenceSystem DEFAULT_CRS = ViewportModelImpl.getDefaultCRS();

    /**
     * Returns the local coordinate system. The local coordinate system is the CRS that all the
     * layer data will be transformed into. Once the layer data is transformed into the local CRS
     * then it is transformed for display onto the screen
     * 
     * @return the local coordinate system
     * @uml.property name="cRS"
     * @model unsettable='true'
     */
    public CoordinateReferenceSystem getCRS();

    /**
     * Sets the value of the '{@link net.refractions.udig.project.internal.render.ViewportModel#getCRS <em>CRS</em>}'
     * attribute. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param value the new value of the '<em>CRS</em>' attribute.
     * @see #isSetCRS()
     * @see #unsetCRS()
     * @see #getCRS()
     * @generated
     */
    void setCRS( CoordinateReferenceSystem value );

    /**
     * Unsets the value of the '{@link net.refractions.udig.project.internal.render.ViewportModel#getCRS <em>CRS</em>}'
     * attribute. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see #isSetCRS()
     * @see #getCRS()
     * @see #setCRS(CoordinateReferenceSystem)
     * @generated
     */
    void unsetCRS();

    /**
     * Returns whether the value of the '{@link net.refractions.udig.project.internal.render.ViewportModel#getCRS <em>CRS</em>}'
     * attribute is set. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return whether the value of the '<em>CRS</em>' attribute is set.
     * @see #unsetCRS()
     * @see #getCRS()
     * @see #setCRS(CoordinateReferenceSystem)
     * @generated
     */
    boolean isSetCRS();

    /**
     * Returns the bounding box of the Viewport in world coordinates.
     * <p>
     * Note: Since Envelope is not a UDIG element changes to the bounds envelope object will not
     * raise events. Therefore the bounds should only be modified via the ViewportModel interface
     * </p>
     * 
     * @return the bounding box of the Viewport in world coordinates.
     * @uml.property name="bounds"
     * @model default=""
     */
    public Envelope getBounds();

    /**
     * Sets the value of the '{@link net.refractions.udig.project.internal.render.ViewportModel#getBounds <em>Bounds</em>}'
     * attribute. <!-- begin-user-doc -->
     * <p>
     * The bbox must have a positive width and height and must have a aspect ratio within 0.0000001
     * units of the value returned by {@linkplain #getAspectRatio()} .
     * </p>
     * <p>
     * It is recommended that setHeight() or setWidth() methods are used since they preserve the
     * aspect ratio of the Viewport
     * <p>
     * <!-- end-user-doc -->
     * 
     * @param value the new value of the '<em>Bounds</em>' attribute.
     * @see #getBounds()
     * @generated
     */
    void setBounds( Envelope value );

    /**
     * Sets the viewport's bounding box. The bounding box will be fit to the window based on the MapDisplay so
     * the bounds provided here may not be the final bounds.
     * 
     * @param minx the minimum x value of the new bounding box.
     * @param maxx the maximum x value of the new bounding box.
     * @param miny the minimum y value of the new bounding box.
     * @param maxy the maximum x value of the new bounding box.
     * @throws IllegalArgumentException If the above criteria are not met this exception is thrown.
     * @model
     */
    public void setBounds( double minx, double maxx, double miny, double maxy )
            throws IllegalArgumentException;

    /**
     * Returns the center of the viewport in world coordinates.
     * 
     * @return the center of the viewport in world coordinates
     * @uml.property name="center"
     * @model volatile="true" transient="true"
     */
    public Coordinate getCenter();

    /**
     * Sets the value of the '{@link net.refractions.udig.project.internal.render.ViewportModel#getCenter <em>Center</em>}'
     * attribute. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param value the new value of the '<em>Center</em>' attribute.
     * @see #getCenter()
     * @generated
     */
    void setCenter( Coordinate value );

    /**
     * Returns the Viewport's height in world coordinates.
     * 
     * @return the Viewport's height in world coordinates.
     * @uml.property name="height"
     * @model volatile="true" transient="true"
     */
    public double getHeight();

    /**
     * Sets the value of the '{@link net.refractions.udig.project.internal.render.ViewportModel#getHeight <em>Height</em>}'
     * attribute. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param value the new value of the '<em>Height</em>' attribute.
     * @see #getHeight()
     * @generated
     */
    void setHeight( double value );

    /**
     * Returns the Viewport's width in world coordinates.
     * 
     * @return the Viewport's width in world coordinates.
     * @uml.property name="width"
     * @model volatile="true" transient="true"
     */
    public double getWidth();

    /**
     * Sets the value of the '{@link net.refractions.udig.project.internal.render.ViewportModel#getWidth <em>Width</em>}'
     * attribute. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param value the new value of the '<em>Width</em>' attribute.
     * @see #getWidth()
     * @generated
     */
    void setWidth( double value );

    /**
     * Returns the aspect ratio of the viewport.
     * 
     * @return The aspect ratio of the viewport.
     * @model volatile="true" changeable="false" transient="true"
     */
    public double getAspectRatio();

    /**
     * Gets the Map that contains the current ViewportModel
     * 
     * @return the Map that contains the current ViewportModel
     * @model opposite="viewportModelInternal" many="false"
     */
    public Map getMapInternal();

    /**
     * Sets the value of the '{@link net.refractions.udig.project.internal.render.ViewportModel#getMapInternal <em>Map Internal</em>}'
     * container reference. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param value the new value of the '<em>Map Internal</em>' container reference.
     * @see #getMapInternal()
     * @generated
     */
    void setMapInternal( Map value );

    /**
     * Returns the value of the '<em><b>Render Manager Internal</b></em>' reference. It is
     * bidirectional and its opposite is '{@link net.refractions.udig.project.internal.render.RenderManager#getViewportModelInternal <em>Viewport Model Internal</em>}'.
     * <!-- begin-user-doc -->
     * <p>
     * If the meaning of the '<em>Render Manager</em>' reference isn't clear, there really
     * should be more of a description here...
     * </p>
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>Render Manager Internal</em>' reference.
     * @see #setRenderManagerInternal(RenderManager)
     * @see net.refractions.udig.project.internal.render.RenderPackage#getViewportModel_RenderManagerInternal()
     * @see net.refractions.udig.project.internal.render.RenderManager#getViewportModelInternal
     * @model opposite="viewportModelInternal" resolveProxies="false" transient="true"
     * @generated
     */
    RenderManager getRenderManagerInternal();

    /**
     * Sets the value of the '{@link net.refractions.udig.project.internal.render.ViewportModel#getRenderManagerInternal <em>Render Manager Internal</em>}'
     * reference. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param value the new value of the '<em>Render Manager Internal</em>' reference.
     * @see #getRenderManagerInternal()
     * @generated
     */
    void setRenderManagerInternal( RenderManager value );

    /**
     * Gets up the affine transform that will transform from the world to screen. A convenience
     * method.
     * 
     * @return a transform that maps from real world coordinates to the screen
     * @model
     */
    public AffineTransform worldToScreenTransform();

    /**
     * Returns the pixel on the screen for a given coordinate in world space.
     * 
     * @param coord A coordinate in world space.
     * @return The pixel on the screen that the world coordinate is drawn on.
     * @model
     */
    public Point worldToPixel( Coordinate coord );

    /**
     * Converts a coordinate expressed on the device space back to real world coordinates
     * 
     * @param x horizontal coordinate on device space
     * @param y vertical coordinate on device space
     * @return The correspondent real world coordinate
     * @model
     */
    public Coordinate pixelToWorld( int x, int y );

    /**
     * Returns the size of a pixel in world units.
     * <p>For example if the world is in WGS 84(lat long) then 
     * the size will be in degrees
     * </p>
     * @return the size of a pixel in world units. 
     * @model volatile="true" changeable="false" transient="true"
     */
    public Coordinate getPixelSize();

    /**
     * Pans the viewport in terms of pixels on the screen. Each pixel represents a distance in world
     * coordinates, the x and y distances differ, so a pan of 8 pixels in the x direction will be
     * translated to a pan of 8*xdistance in the world.
     * 
     * @param xpixels The amount to pan in the x direction
     * @param ypixels The amount to pan in the y direction
     * @return this ViewportModel
     * @model
     */
    public ViewportModel panUsingScreenCoords( int xpixels, int ypixels );

    /**
     * Pans the viewport in terms of world units.
     * 
     * @param x The amount to pan in the x direction
     * @param y The amount to pan in the y direction
     * @return this ViewportModel
     * @model
     */
    public ViewportModel panUsingWorldCoords( double x, double y );

    /**
     * Increases or decreases the size of the viewport(in world space) by a constant factor, zoom.
     * The zoom is equal in both directions. The function used is: bbox.height=bbox.height/divisor
     * bbox.width=bbox.width/divisor
     * <ul>
     * <li>A zoom must be greater than 1.</li>
     * <li>A zoom greater than 1 is a zoom towards the map(Feature appear larger.)</li>
     * <li>A zoom less than 1 is a zoom away from the map</li>
     * </ul>
     * 
     * @param zoom the zoom factor
     * @return This ViewportModel, allows for command chaining.
     * @model
     */
    public ViewportModel zoom( double zoom );

    /**
     * sets the Viewport bounding box so that it fully contains the visible map extent
     * 
     * @model
     */
    public void zoomToExtent();

    /**
     * sets the Viewport bounding box so that it contains the new box
     * 
     * @model
     */
    public void zoomToBox( Envelope box );

    /**
     * Return true if it has been initialized with a display. If no display is active then the
     * viewportmodel is not initialized.
     * 
     * @return true if it has been initialized with a display
     * @uml.property name="initialized"
     */
    public boolean isInitialized();

    /**
     * Sets the initialized state of ViewportModel.
     * 
     * @param initialized
     * @uml.property name="initialized"
     */
    public void setInitialized( boolean initialized );
    
    /**
     * Sets the zoom level of the viewport so that the scale denominator will be equal to
     * scale
     *
     * @param scaleDenominator desired scale denominator 
     */
    public void setScale(double scaleDenominator);

    
}
