/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.ui.internal.properties;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource2;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import com.vividsolutions.jts.geom.Coordinate;

/**
 * A Property source for coordinate objects
 * 
 * @author jeichar
 * @since 0.3
 */
public class CoordinatePropertySource implements IPropertySource2 {
    private Coordinate coord;
    private Coordinate old;
    private TextPropertyDescriptor[] descriptors;
    /**
     * Creates a new instance of CoordinatePropertyDescriptor
     * 
     * @param coordinate
     */
    public CoordinatePropertySource( Coordinate coordinate ) {
        this.coord = coordinate;
        old = new Coordinate(coordinate);
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
     */
    public Object getEditableValue() {
        return null;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
    public IPropertyDescriptor[] getPropertyDescriptors() {
        if (descriptors == null) {
            descriptors = new TextPropertyDescriptor[]{new TextPropertyDescriptor("x", "x"), //$NON-NLS-1$ //$NON-NLS-2$
                    new TextPropertyDescriptor("y", "y"), //$NON-NLS-1$ //$NON-NLS-2$
                    new TextPropertyDescriptor("z", "z"), //$NON-NLS-1$ //$NON-NLS-2$
            };
        }

        IPropertyDescriptor[] c=new IPropertyDescriptor[descriptors.length];
        System.arraycopy(descriptors, 0, c, 0, c.length);
        return c;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
     */
    public Object getPropertyValue( Object id ) {
        if (id.equals("x")) //$NON-NLS-1$
            return String.valueOf(coord.x);
        if (id.equals("y")) //$NON-NLS-1$
            return String.valueOf(coord.y);
        if (id.equals("z")) //$NON-NLS-1$
            return String.valueOf(coord.z);
        return null;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
     */
    public boolean isPropertySet( Object id ) {
        if (id.equals("x")) //$NON-NLS-1$
            return old.x == coord.x;
        if (id.equals("y")) //$NON-NLS-1$
            return old.x == coord.y;
        if (id.equals("z")) //$NON-NLS-1$
            return old.x == coord.z;
        return false;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
     */
    public void resetPropertyValue( Object id ) {
        if (id.equals("x")) //$NON-NLS-1$
            coord.x = old.x;
        else if (id.equals("y")) //$NON-NLS-1$
            coord.y = old.y;
        else if (id.equals("z")) //$NON-NLS-1$
            coord.z = old.z;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
     *      java.lang.Object)
     */
    public void setPropertyValue( Object id, Object value ) {
        if (id.equals("x")) //$NON-NLS-1$
            coord.x = new Double((String) value).doubleValue();
        else if (id.equals("y")) //$NON-NLS-1$
            coord.y = new Double((String) value).doubleValue();
        else if (id.equals("z")) //$NON-NLS-1$
            coord.z = new Double((String) value).doubleValue();

    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource2#isPropertyResettable(java.lang.Object)
     */
    public boolean isPropertyResettable( Object id ) {
        // TODO
        return true;
    }
}
