/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.tool.info.internal;

import java.io.IOException;
import java.util.List;

import net.refractions.udig.project.AdaptableFeature;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.impl.LayerImpl;
import net.refractions.udig.project.ui.ApplicationGIS;
import net.refractions.udig.project.ui.internal.properties.FeaturePropertySource;
import net.refractions.udig.tool.info.InfoPlugin;
import net.refractions.udig.tool.info.InfoTool;
import net.refractions.udig.tool.info.LayerPointInfo;
import net.refractions.udig.tool.info.internal.display.BrowserInfoDisplay;
import net.refractions.udig.tool.info.internal.display.TextInfoDisplay;
import net.refractions.udig.ui.SearchPart;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.geotools.data.FeatureSource;
import org.geotools.data.ows.Layer;
import org.geotools.feature.Feature;
import org.geotools.geometry.jts.ReferencedEnvelope;

/**
 * Info recast as a search part.
 * 
 * @author jgarnett
 * @since 1.0.0
 */
public class InfoView2 extends SearchPart {
    /** <code>VIEW_ID</code> field */
    public static final String VIEW_ID = "net.refractions.udig.tool.info.infoView"; //$NON-NLS-1$
    private Text information;
    private TextInfoDisplay textDisplay;
    private BrowserInfoDisplay browserDisplay;
    //private FeatureDisplay featureDisplay;
    private PropertySheetPage featureDisplay;
    
    private static final class InfoViewLabelProvider extends LabelProvider implements IColorProvider {
        public String getText(Object element) {
            if (element instanceof Feature) {
                return ((Feature) element).getID();
            }
            else if (element instanceof LayerPointInfo) {
                LayerPointInfo info = (LayerPointInfo) element;                    
                return info.getLayer().getName();
            }
            return super.getText(element);
        }
        
        
        /**
         * @see net.refractions.udig.project.internal.provider.LayerItemProvider
         */
        public Color getBackground(Object element) {
            if (element instanceof AdaptableFeature) {
                AdaptableFeature feature = (AdaptableFeature) element;
                LayerImpl layer = (LayerImpl) feature.getAdapter(ILayer.class);
                IColorProvider colorProvider = (IColorProvider) layer.getAdapter(IColorProvider.class);
                if (colorProvider == null) return null;
                return colorProvider.getBackground(layer);
            }            
            return null;
        }

        /**
         * @see net.refractions.udig.project.internal.provider.LayerItemProvider
         */
        public Color getForeground(Object element) {
            if (element instanceof AdaptableFeature) {
                AdaptableFeature feature = (AdaptableFeature) element;
                LayerImpl layer = (LayerImpl) feature.getAdapter(ILayer.class);
                IColorProvider colorProvider = (IColorProvider) layer.getAdapter(IColorProvider.class);
                if (colorProvider == null) return null;
                return colorProvider.getForeground(layer);
            }
            return null;
        }
    }

    public static class InfoRequest {
        public ReferencedEnvelope bbox;
        public List<ILayer> layers;
    };
    
    /**
     * 
     */
    public InfoView2() {
        super( InfoPlugin.getDefault().getDialogSettings() );
    }
    
    @Override
    protected Composite createDetails( SashForm splitter ) {
        PageBook book = new PageBook( splitter, SWT.NONE );        
        splitter.setWeights(new int[]{10, 90});

        {
            information = new Text(book, SWT.WRAP );
            information.setText(Messages.InfoView_instructions_text); 
            book.showPage(information);            
        }
        {
            textDisplay = new TextInfoDisplay();
            textDisplay.createDisplay(book);
            //textDisplay.getControl().setVisible(false);
        }
        {
            browserDisplay = new BrowserInfoDisplay();
            browserDisplay.createDisplay(book);
            //browserDisplay.getControl().setVisible(false);
        }
        {
            //featureDisplay = new FeatureDisplay();
            //featureDisplay.createDisplay(book);
            featureDisplay = new PropertySheetPage();
            featureDisplay.createControl(book);
            //featureDisplay.getControl().setVisible(false);
        }              
        return book;
    }
    @Override    
    protected void fillActionBars() {
        IActionBars actionBars = getViewSite().getActionBars();
        IToolBarManager toolBar = actionBars.getToolBarManager();
     
        IAction infoTool = ApplicationGIS.getToolManager().getTool(InfoTool.ID, InfoTool.CATEGORY_ID);
        assert( infoTool != null );
        if( toolBar != null ){
            toolBar.add( infoTool );
        }
        super.fillActionBars();
        
    }
    
    @Override
    protected PageBook getDetails() {
        return (PageBook) super.getDetails();
    }
    @Override
    protected void showDetail( Object selection ) {
        if (selection == null ) {
            getDetails().showPage(information);
            return;
        }
        if( selection instanceof Feature ) {
            getDetails().showPage(featureDisplay.getControl());
            FeaturePropertySource source = new FeaturePropertySource( (Feature)selection, false, false);
            StructuredSelection sel = new StructuredSelection(source);
            featureDisplay.selectionChanged(null, sel);
            //featureDisplay.setInfo( (Feature) selection );
        }
        if (selection instanceof LayerPointInfo) {
            LayerPointInfo info = (LayerPointInfo) selection;
            if (info.getMimeType() == null) {
                getDetails().showPage(information);
            } else if (info.getMimeType().startsWith(LayerPointInfo.GML)) {
                getDetails().showPage(featureDisplay.getControl());
                try {
                    Feature feature = (Feature) info.acquireValue();
                    FeaturePropertySource src = new FeaturePropertySource(feature);
                    StructuredSelection sel = new StructuredSelection(src);
                    featureDisplay.selectionChanged(null, sel);
                } catch(IOException ex) {
                    InfoPlugin.log("GML value could not be acquired.", ex); //$NON-NLS-1$
                }
                //featureDisplay.setInfo(info);
            } else if (info.getRequestURL() != null &&
                       info.getMimeType().startsWith(LayerPointInfo.HTML)) {
                getDetails().showPage( browserDisplay.getControl() );
                browserDisplay.setInfo(info);        
            } else if (info.getRequestURL() != null && info.getMimeType().startsWith(LayerPointInfo.XML)) {
                getDetails().showPage( browserDisplay.getControl() );
                browserDisplay.setInfo(info);
            } else if (info.getRequestURL() != null && info.getMimeType().startsWith(LayerPointInfo.ERROR)) {
                getDetails().showPage( browserDisplay.getControl() );
                browserDisplay.setInfo(info);
            } else {
                getDetails().showPage(information);
            }
        }        
    
    }
    
    @Override
    public void createPartControl( Composite aParent ) {
        super.createPartControl(aParent);
        ApplicationGIS.getToolManager().registerActionsWithPart(this);
    }
    
    /**
     * Called by search( Object filter ) in a job to get stuff done.
     * <p>
     * Resulting like is provided to the viewer via setInput.
     * </p>
     */
    @Override
    protected void searchImplementation( Object filter, IProgressMonitor monitor, ResultSet set ) {
        InfoRequest request = (InfoRequest) filter;
            
        if( monitor != null ) {
            monitor.beginTask( Messages.InfoView2_information_request, request.layers.size() );
        }
        int work = 0;
        for( int i = request.layers.size()-1; i > -1; i-- ) {
            ILayer layer = request.layers.get(i); //navigate the list backwards
            monitor.subTask( layer.getName() );
            monitor.worked( ++work );
            
            if( !layer.isVisible() ) continue;
            //if( !layer.isApplicable( "information" )) continue;
            
            if( layer.hasResource( FeatureSource.class ) ) {
                try {
                    List<Feature> more = DataStoreDescribeLayer.info2( layer, request.bbox, monitor );
                    if( !more.isEmpty() ) set.addAll( more );
                }
                catch( Throwable t ) {
                    InfoPlugin.log( "Information request "+layer.getName()+" failed "+t, t ); //$NON-NLS-1$ //$NON-NLS-2$
                }
                continue;
            }            
            if( layer.hasResource( Layer.class ) ) {
                // TODO: freek out because WMS is so hard to use
                try {
                    LayerPointInfo hit = WMSDescribeLayer.info2( layer, request.bbox );
                    if( hit != null ) set.add( hit );
                }
                catch( Throwable t ) {
                    InfoPlugin.log( "Information request "+layer.getName()+" failed "+t, t ); //$NON-NLS-1$ //$NON-NLS-2$
                }
                continue;
            }
        }
    }
    @Override
    protected IBaseLabelProvider createLabelProvider() {
        return new InfoViewLabelProvider();
    }
  
    @Override
    protected StructuredViewer createViewer( Composite parent ) {
        TreeViewer viewer = new TreeViewer(parent);
        return viewer;
    }

    @Override
    protected ISelection getSelection( List results ) {

        for( Object object : results ) {
            ILayer layer=null ;
            if( object instanceof LayerPointInfo ){
                LayerPointInfo info=(LayerPointInfo) object;
                layer = info.getLayer();
            }
            if( object instanceof AdaptableFeature ){
                AdaptableFeature feature=(AdaptableFeature) object;
                layer=(ILayer) feature.getAdapter(ILayer.class);
            }
            if( layer==ApplicationGIS.getActiveMap().getEditManager().getSelectedLayer() ){
                return new StructuredSelection(object);
            }
        }
        if( results.isEmpty() )
            return new StructuredSelection();
        
        return new StructuredSelection(results.get(0));
    }

    @Override
    protected IStructuredContentProvider createContentProvider() {
        return new ITreeContentProvider() {
            public Object[] getElements( Object inputElement ) {
                if( inputElement instanceof List ) {
                    return ((List)inputElement).toArray();
                }
                return null;
            }
            public void dispose() {
            }
            public void inputChanged( Viewer viewer, Object oldInput, Object newInput ) {
                //assert( newInput instanceof List );
                // lists don't have events for us to watch
            }
            public Object[] getChildren( Object parentElement ) {
                return null;
            }
            public Object getParent( Object element ) {
                return null;
            }
            public boolean hasChildren( Object element ) {
                return false;
            }            
        };
    }
}