/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.ui.graphics;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;

import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.Path;


public class AWTGraphics implements ViewportGraphics {

    public Graphics2D g;

    public AWTGraphics( Graphics2D g ) {
        this.g = g;
        g.setBackground(Color.WHITE);
    }

    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#draw(java.awt.Shape)
     */
    public void draw( Shape s ) {
        g.draw(s);
    }
    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#draw(java.awt.Shape)
     */
    public void fill( Shape s ) {
        g.fill(s);
    }
    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#setColor(java.awt.Color)
     */
    public void setColor( Color c ) {
        g.setColor(c);
    }

    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#setBackground(java.awt.Color)
     */
    public void setBackground( Color c ) {
        g.setBackground(c);        
    }


    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#setClip(java.awt.Rectangle)
     */
    public void setClip( Rectangle r ) {
        g.setClip(r);
    }

    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#fillRect(int, int, int, int)
     */
    public void fillRect( int x, int y, int width, int height ) {
        g.fillRect(x, y, width, height);
    }
    
    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#translate(java.awt.Point)
     */
    public void translate( Point offset ) {
        g.setTransform(AffineTransform.getTranslateInstance(offset.x, offset.y));
    }

    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#clearRect(int, int, int, int)
     */
    public void clearRect( int x, int y, int width, int height ) {
        g.clearRect(x,y,width,height);
    }

    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#drawImage(javax.media.jai.PlanarImage, int, int)
     */
    public void drawImage( RenderedImage image, int x, int y ) {
        g.drawRenderedImage(image, AffineTransform.getTranslateInstance(x,y));
    }

    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#drawString(String, int, int)
     */
    public void drawString(String string,int x,int y, int alignx, int aligny) {
        Rectangle2D text = g.getFontMetrics().getStringBounds( string, g );
        int w = (int)text.getWidth();
        int h = (int)text.getHeight();
        
        int x2 = (alignx == 0) ? x - w/2 : (alignx > 0) ? x - w : x;
        int y2 = (aligny == 0) ? y + h/2 : (aligny > 0) ? y + h : y;
        g.drawString(string,x2,y2);
    }
    
    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#setTransform(java.awt.geom.AffineTransform)
     */
    public void setTransform( AffineTransform transform ) {
        g.setTransform(transform);
        
    }

	/**
	 * @see net.refractions.udig.ui.graphics.ViewportGraphics#drawImage(java.awt.Image, int, int)
	 */
	public void drawImage(Image image, int x, int y) {
		g.drawImage(image, x,y,null);
	}

	public void drawImage(Image image, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2) {
		g.drawImage(image, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, null );
	}
    
    public int getFontHeight() {
        return g.getFontMetrics().getHeight();
    }
    
    public int stringWidth(String str) {
        return g.getFontMetrics().stringWidth(str);
    }
    
    public int getFontAscent() {
        return g.getFontMetrics().getAscent();
    }
        
    public Rectangle2D getStringBounds(String str) {
        return g.getFontMetrics().getStringBounds(str, g);
    }
    
    /**
     * Converts an SWT image to an AWT BufferedImage
     * 
     * @param swtImageData
     * @return
     */
    public static BufferedImage toAwtImage(ImageData swtImageData) {
        int width = swtImageData.width;
        int height = swtImageData.height;
        PaletteData palette = swtImageData.palette;
        BufferedImage bufferedImage = new BufferedImage(width, height,
                          BufferedImage.TYPE_INT_RGB);
        for (int y = 0; y < height; y++) {
            for (int x = 0; x < width; x++) {
                int rgb = swtImageData.getPixel(x, y);

                int red = (rgb & palette.redMask) >> Math.abs(palette.redShift);
                int green = (rgb & palette.greenMask) >> Math.abs(palette.greenShift);
                int blue = (rgb & palette.blueMask) >>> Math.abs(palette.blueShift);
                
                Color color = new Color(red, green, blue);                
                bufferedImage.setRGB(x, y, color.getRGB());
            }
        }
        return bufferedImage;
    }

    public void drawLine( int x1, int y1, int x2, int y2 ) {
        g.drawLine(x1, y1, x2, y2);
    }

	public void drawImage(org.eclipse.swt.graphics.Image image, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2) {
		BufferedImage awtImage = toAwtImage(image.getImageData());
        drawImage(awtImage, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2);
	}

	public AffineTransform getTransform() {
		return g.getTransform();
	}

	public void dispose() {
		g.dispose();
	}

    public void drawPath( Path path ) {
        throw new UnsupportedOperationException("We have not had time to implement this method yet"); //$NON-NLS-1$
    }

    public void fillPath( Path path ) {
        throw new UnsupportedOperationException("We have not had time to implement this method yet"); //$NON-NLS-1$
    }

    public void drawRect( int x, int y, int width, int height ) {
        g.drawRect(x,y,width, height);
    }
    
    public void drawOval( int x, int y, int width, int height ){
        g.drawOval(x,y,width, height);
    }

    public void fillOval( int x, int y, int width, int height ) {
        g.fillOval(x, y, width, height);
    }

    public void drawImage( org.eclipse.swt.graphics.Image swtImage, int x, int y ) {
        BufferedImage awtImage = toAwtImage(swtImage.getImageData());
        drawImage((Image)awtImage, x, y);

    }

    public Shape getClip() {
        return g.getClip();
    }

    public void setClipBounds( Rectangle newBounds ) {
        g.setClip(newBounds);
    }

	public Color getBackgroundColor() {
		return g.getBackground();
	}

	public Color getColor() {
		return g.getColor();
	}

    public void drawRoundRect( int x, int y, int width, int height, int arcWidth, int arcHeight ) {
        g.drawRoundRect(x, y, width, height, arcWidth, arcHeight);
    }

    public void fillRoundRect( int x, int y, int width, int height, int arcWidth, int arcHeight ) {
        g.fillRoundRect(x, y, width, height, arcWidth, arcHeight);
    }

    public void setLineDash( int[] dash ) {
        Stroke stroke = g.getStroke();
        if (!(stroke instanceof BasicStroke)) {
            stroke = new BasicStroke();
        }
        BasicStroke basicStroke = (BasicStroke) stroke;
        g.setStroke(new BasicStroke(basicStroke.getLineWidth(), basicStroke.getEndCap(),
                basicStroke.getLineJoin(), basicStroke.getMiterLimit(), toFloatArray(dash), 0));
    }

    private float[] toFloatArray( int[] dash ) {
        float[] result = new float[dash.length];
        for( int i = 0; i < result.length; i++ ) {
            result[i]=dash[i];
        }
        return result;
    }

    public void setLineWidth( int width ) {
        Stroke stroke = g.getStroke();
        if (!(stroke instanceof BasicStroke)) {
            stroke = new BasicStroke();
        }
        BasicStroke basicStroke = (BasicStroke) stroke;
        g.setStroke(new BasicStroke(width, basicStroke.getEndCap(),
                basicStroke.getLineJoin(), basicStroke.getMiterLimit(), basicStroke.getDashArray(),
                basicStroke.getDashPhase()));

    }

    /**
     * @see net.refractions.udig.ui.graphics.ViewportGraphics#setStroke(int, int)
     */
    public void setStroke( int style, int width ) {
        switch( style ) {
        case LINE_DASH: {
            g.setStroke(new BasicStroke(width, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER,
                    10.0f, new float[]{width * 2.0f, width * 2.0f}, 0.0f));
            break;
        }
        case LINE_DASHDOT: {
            g.setStroke(new BasicStroke(width, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER,
                    10.0f, new float[]{width * 2.0f, width * 2.0f, width * 1.0f, width * 2.0f},
                    0.0f));
            break;
        }
        case LINE_DASHDOTDOT: {
            g.setStroke(new BasicStroke(width, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER,
                    10.0f, new float[]{width * 2.0f, width * 2.0f, width * 1.0f, width * 2.0f,
                            width * 1.0f, width * 2.0f}, 0.0f));
            break;
        }
        case LINE_DOT: {
            g.setStroke(new BasicStroke(width, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER,
                    10.0f, new float[]{width * 1.0f, width * 2.0f}, 0.0f));
            break;
        }
        case LINE_SOLID: {
            g.setStroke(new BasicStroke(width));
            break;
        }
        }
    }
}